@@ -5,11 +5,11 @@
  * Uses reservoir sampling to break ties at random.
  */
 
-#include "R_ext/Arith.h" /* NA handling */
-#include <Rmath.h> /* fmax2 */
-#include "R_ext/Random.h" /* ..RNGstate */
+#include <R_ext/Arith.h>	/* NA handling */
+#include <Rmath.h>		/* fmax2 */
+#include <R_ext/Random.h>	/* ..RNGstate */
 
-#include "R_ext/Applic.h" /* NA handling */
+#include <R_ext/Applic.h>	/* NA handling */
 
 #define RELTOL 1e-5
 
