@@ -218,8 +218,8 @@ static void xbuffixl(xbuf p)
 
 /* console */
 
-rgb consolebg = White, consolefg = Black, consoleuser = Red,
-    pagerhighlight = Red;
+rgb consolebg = White, consolefg = Black, consoleuser = gaRed,
+    pagerhighlight = gaRed;
 
 extern int R_HistorySize;  /* from Defn.h */
 
@@ -516,7 +516,11 @@ FBEGIN
 FVOIDEND
 
 
-/* These are the getline keys ^A ^E ^B ^F ^N ^P ^K ^H ^D ^U ^T ^O */
+/* These are the getline keys ^A ^E ^B ^F ^N ^P ^K ^H ^D ^U ^T ^O,
+   plus ^Z for EOF.
+
+   We also use ^C ^V/^Y ^X (copy/paste/both) ^W ^L
+*/
 #define BEGINLINE 1
 #define ENDLINE   5
 #define CHARLEFT 2
@@ -529,7 +533,8 @@ FVOIDEND
 #define KILLLINE 21
 #define CHARTRANS 20
 #define OVERWRITE 15
-/* free ^G ^Q ^R ^S */
+#define EOFKEY 26
+/* free ^G ^Q ^R ^S, perhaps ^I ^J */
 
 static void storekey(control c,int k)
 FBEGIN
@@ -1064,7 +1069,7 @@ FBEGIN
 		cur_line[cur_pos-1] = cur_char;
 		break;
 	    default:
-		if (chtype || (cur_char=='\n')) {
+		if (chtype || (cur_char=='\n') || (cur_char==EOFKEY)) {
 		    if (chtype) {
 			if (cur_pos == max_pos) {
 			    consoleunputc(c);
@@ -1082,7 +1087,7 @@ FBEGIN
 		    xbuffixl(p->lbuf);
 		    consolewrites(c, "\n");
 		    REDRAW;
-		    FRETURN(0);
+		    FRETURN(cur_char == EOFKEY);
 		}
 		break;
 	    }
