@@ -2,17 +2,16 @@ barplot <- function(height, ...) UseMethod("barplot")
 
 barplot.default <-
 function(height, width = 1, space = NULL, names.arg = NULL,
-         legend.text = NULL, beside = FALSE, horiz = FALSE,
-         density = NULL, angle = 45,
-         col = heat.colors(NR), border = par("fg"),
-         main = NULL, sub = NULL, xlab = NULL, ylab = NULL,
-         xlim = NULL, ylim = NULL, xpd = TRUE,
-         axes = TRUE, axisnames = TRUE,
-         cex.axis = par("cex.axis"), cex.names = par("cex.axis"),
-         inside = TRUE, plot = TRUE, ...)
+	 legend.text = NULL, beside = FALSE, horiz = FALSE,
+	 density = NULL, angle = 45,
+	 col = heat.colors(NR), border = par("fg"),
+	 main = NULL, sub = NULL, xlab = NULL, ylab = NULL,
+	 xlim = NULL, ylim = NULL, xpd = TRUE,
+	 axes = TRUE, axisnames = TRUE,
+	 cex.axis = par("cex.axis"), cex.names = par("cex.axis"),
+	 inside = TRUE, plot = TRUE, axis.lty = 0, ...)
 {
-    if (!missing(inside)) .NotYetUsed("inside", error = FALSE)
-    if (!missing(border)) .NotYetUsed("border", error = FALSE)
+    if (!missing(inside)) .NotYetUsed("inside", error = FALSE)# -> help(.)
 
     if (missing(space))
 	space <- if (is.matrix(height) && beside) c(0, 1) else 0.2
@@ -31,12 +30,9 @@ function(height, width = 1, space = NULL, names.arg = NULL,
     } else if (!is.matrix(height))
 	stop("`height' must be a vector or a matrix")
 
-    if(is.logical(legend.text)) {
-        if(legend.text && is.matrix(height))
-            legend.text <- rownames(height)
-        else
-            legend.text <- NULL
-    }
+    if(is.logical(legend.text))
+	legend.text <-
+	    if(legend.text && is.matrix(height)) rownames(height)
 
     NR <- nrow(height)
     NC <- ncol(height)
@@ -70,19 +66,20 @@ function(height, width = 1, space = NULL, names.arg = NULL,
 
 	plot.new()
 	plot.window(xlim, ylim, log = "", ...)
-        # Beware : angle and density are passed using R scoping rules
 	xyrect <- function(x1,y1, x2,y2, horizontal = TRUE, ...) {
 	    if(horizontal)
-		rect(x1,y1, x2,y2, angle = angle, density = density, ...)
+		rect(x1,y1, x2,y2, ...)
 	    else
-		rect(y1,x1, y2,x2, angle = angle, density = density, ...)
+		rect(y1,x1, y2,x2, ...)
 	}
 	if (beside)
-          xyrect(0, w.l, c(height), w.r, horizontal=horiz, col = col)
+	    xyrect(0, w.l, c(height), w.r, horizontal = horiz,
+		   angle = angle, density = density, col = col, border = border)
 	else {
 	    for (i in 1:NC) {
 		xyrect(height[1:NR, i], w.l[i], height[-1, i], w.r[i],
-		       horizontal=horiz, col = col)
+		       horizontal = horiz, angle = angle, density = density,
+		       col = col, border = border)
 	    }
 	}
 	if (axisnames && !is.null(names.arg)) { # specified or from {col}names
@@ -92,21 +89,21 @@ function(height, width = 1, space = NULL, names.arg = NULL,
 		else
 		    stop("incorrect number of names")
 	    } else w.m
-	    axis(if(horiz) 2 else 1, at = at.l,
-                 labels = names.arg, lty = 0, cex.axis = cex.names, ...)
+	    axis(if(horiz) 2 else 1, at = at.l, labels = names.arg,
+		 lty = axis.lty, cex.axis = cex.names, ...)
 	}
 	if(!is.null(legend.text)) {
 	    legend.col <- rep(col, length = length(legend.text))
 	    if((horiz & beside) || (!horiz & !beside)){
 		legend.text <- rev(legend.text)
 		legend.col <- rev(legend.col)
-                density <- rev(density)
-                angle <- rev(angle)
+		density <- rev(density)
+		angle <- rev(angle)
 	    }
 	    xy <- par("usr")
 	    legend(xy[2] - xinch(0.1), xy[4] - yinch(0.1),
 		   legend = legend.text, angle = angle, density = density,
-                   fill = legend.col, xjust = 1, yjust = 1)
+		   fill = legend.col, xjust = 1, yjust = 1)
 	}
 	title(main = main, sub = sub, xlab = xlab, ylab = ylab, ...)
 	if(axes) axis(if(horiz) 1 else 2, cex.axis = cex.axis, ...)
