@@ -1,8 +1,9 @@
-sapply <- function(X, FUN, ..., simplify = TRUE)
+sapply <- function(X, FUN, ..., simplify = TRUE, USE.NAMES = TRUE)
 {
     FUN <- match.fun(FUN)
-
     answer <- lapply(as.list(X), FUN, ...)
+    if(USE.NAMES && is.character(X) && is.null(names(answer)))
+                names(answer) <- X
     if(simplify && length(answer) &&
        length(common.len <- unique(unlist(lapply(answer, length)))) == 1) {
 	if(common.len == 1)
