@@ -2,9 +2,10 @@ tabulate <- function(bin, nbins = max(1,bin))
 {
     if(!is.numeric(bin) && !is.factor(bin))
 	stop("tabulate: bin must be numeric or a factor")
-    .C("tabulate",
+    .C("R_tabulate",
        as.integer(bin),
        as.integer(length(bin)),
        as.integer(nbins),
-       ans = integer(nbins))$ans
+       ans = integer(nbins),
+       PACKAGE="base")$ans
 }
