@@ -1,16 +1,44 @@
 setClass <-
-  ## Define Class to be an S-style class.
-  function(Class, representation = list(), prototype = NULL,
-                     contains = character(), validity = NULL, access = NULL,
-                     where = 1, version = FALSE)
+    ## Define Class to be an S-style class.
+    function(Class, representation = list(), prototype = NULL,
+             contains = character(), validity = NULL, access = list(),
+             where = 1, version = .newExternalptr(), sealed = FALSE, package = getPackageName(where))
 {
-    if(!is.na(match(Class, .BasicClasses)))
-        stop(paste("\"", Class, "\" is a basic class and cannot be redefined", sep=""))
-    slots <- nchar(allNames(representation))>0
-    extends <- c(as.character(representation[!slots]), contains)
-    properties <- representation[slots]
-    setSClass(Class, properties,extends, prototype, where=where,
-              validity = validity, access = access)
+    if(isClass(Class) && getClassDef(Class)@sealed)
+        stop(paste("\"", Class, "\" has a sealed class definition and cannot be redefined", sep=""))
+    if(is(representation, "classRepresentation")) {
+        ## supplied a class definition object
+        classDef <- representation
+        if(!(missing(prototype) && missing(contains) && missing(validity) && missing(access)
+             && missing(version) && missing(package)))
+            stop("Only arguments Class and where can be supplied when argument \"representation\" is a classRepresentation object")
+        if(length(classDef@package)==0)
+            classDef@package <- package # the default
+        superClasses <- allNames(classDef@contains)
+    }
+    else { 
+        ## catch the special case of a single class name as the representation
+        if(is.character(representation) && length(representation) == 1 &&
+           is.null(names(representation)))
+            representation <- list(representation)
+        slots <- nchar(allNames(representation))>0
+        superClasses <- c(as.character(representation[!slots]), contains)
+        properties <- representation[slots]
+        classDef <- makeClassRepresentation(Class, properties,superClasses, prototype, package,
+                                             validity, access, version, sealed)
+        superClasses <- names(classDef@contains)
+    }
+    assignClassDef(Class, classDef, where)
+    for(class2 in superClasses)
+        setIs(Class, class2, where = where)
+    ## confirm the validity of the class definition (it may be incomplete)
+    msg <- trySilent(completeClassDefinition(Class, classDef))
+    if(is(msg, "try-error")) {
+        removeClass(Class, where=where)
+        warning("Cannot complete class definition for \"", Class, "\" (",
+                   msg, ") -- may conflict with previous definitions")
+    }
+    Class
 }
 
 representation <-
@@ -33,48 +61,72 @@ representation <-
         stop(paste("Duplicate class names among superclasses:", paste(includes[duplicated(includes)], collapse = ", ")), sep="")
     slots <- anames[nchar(anames)>0]
     if(any(duplicated(slots)))
-       stop(paste("Duplicated slot names: ", paste(slots[duplicated(slots)], collapse="")), sep="")
+       stop(paste("Duplicated slot names: ", paste(slots[duplicated(slots)], collapse=""), sep=""))
     value
 }
 
-setSClass <-
+### the version called prototype is the external interface.  But functions with argument
+### named prototype in R cannot call the prototype function (until there is a methods namespace
+### to allow methods::prototype(...)
+prototype <- function(...)
+    .prototype(...)
+
+.prototype <- function(...) {
+    props <- list(...)
+    names <- allNames(props)
+    data <- nchar(names) == 0
+    if(any(data)) {
+        if(sum(data) > 1)
+            stop("only one data object (unnamed argument to prototype) allowed")
+        obj <- props[[seq(along=data)[data] ]]
+        props <- props[!data]
+        names <- names[!data]
+    }
+    else
+        obj <- list()
+    for(i in seq(along = names))
+        slot(obj, names[[i]], FALSE) <- props[[i]]
+    new("classPrototypeDef", object = obj, slots = names)
+}
+
+makeClassRepresentation <-
   ## Set the Class Definition.
   ## The formal definition of the class is set according to the arguments.
   ##
   ## Users should call setClass instead of this function.
-  function(name, properties = list(), extends = character(), prototype = NULL, generatorFunction, where = 1, subclasses = character(), virtual = NA, validity = NULL, access = NULL)
+  function(name, slots = list(), superClasses = character(), prototype = NULL, package, validity = NULL, access = list(), version = .newExternalptr(), sealed = FALSE, virtual = NA)
 {
     ## remove from the cached definition (only) right away
     removeClass(name, where = 0)
-    extends <- makeExtends(extends)
-    subclasses <- makeExtends(subclasses)
-    if(!is.null(prototype) || length(properties)>0 || length(extends) >0) {
+    if(!is.null(prototype) || length(slots)>0 || length(superClasses) >0) {
         ## collect information about slots, create prototype if needed
-        pp <- reconcilePropertiesAndPrototype(name, properties, prototype, extends)
-        properties <- pp$properties
+        pp <- reconcilePropertiesAndPrototype(name, slots, prototype, superClasses)
+        slots <- pp$properties
         prototype <- pp$prototype
-        extends <- pp$extends
     }
-    ev <- newClassEnvironment(name, properties, extends, prototype, subclasses,
-                              virtual, validity, access)
-    if(missing(generatorFunction)) {
-        f <- substitute(function(...)newObject(Class=CLASS, ...), list(CLASS=name))
+    contains <- list();
+    for(what in superClasses) {
+        whatClassDef <- getClass(what)
+        ## Create the SClassExtension objects (will be simple, possibly dataPart).
+        ## The slots are supplied explicitly, since `name' is currently an undefined class
+        elNamed(contains, what) <- makeExtends(name, what, slots = slots,
+                                              classDef2 = whatClassDef)
     }
-    else {
-        assign(".Generator", generatorFunction, ev)
-        f <- quote(function(...).Generator(...))
+    validity <- .makeValidityMethod(name, validity)
+    if(is.na(virtual)) {
+        virtual <- testVirtual(slots, contains, prototype)
+        if(virtual && !is.na(match("VIRTUAL", superClasses)))
+            elNamed(contains, "VIRTUAL") <- NULL
     }
-    mode(f) <- "function"               ## because mode is call otherwise
-    class(f) <- "classRepEnvironment"
-    environment(f) <- ev
-    setValidity(f, validity)
-    assignClassDef(name, f, where)
-    ## confirm the validity of the class definition (it may be incomplete)
-    on.exit(removeClassDef(name, where))
-    completeClassDefinition(name, f)
-    ## if no error, allow assignment to stay
-    on.exit()
-    name
+    newClassRepresentation(className = name, slots = slots,
+                           contains = contains,
+                           prototype = prototype,
+                           virtual = virtual,
+                           validity = validity,
+                           access = access,
+                           package = package,
+                           versionKey = version,
+                           sealed = sealed)
 }
 
 getClassDef <-
@@ -108,15 +160,16 @@ getClass <-
 {
     value <- getClassDef(Class, 0)
     if(is.null(value)) {
-        if(isClass(Class)) {
-            value <- completeClassDefinition(Class)
-            assignClassDef(Class, value, 0)
+        value  <- getClassDef(Class)
+        if(is.null(value)) {
+            if(!.Force)
+                stop(paste("\"", Class, "\" is not a defined class", sep=""))
+            else
+                value <- makeClassRepresentation(Class, package = "base", virtual = TRUE)
         }
-        else if(!.Force && is.na(match(Class, .BasicClasses)))
-        stop(paste("\"", Class, "\" is not a defined class", sep=""))
-      ## else, return NULL.  This may change, if we force formal definitions
-      ## for all classes, including (the tough ones) NULL, array, matrix, ts 
-    }
+        else
+            value <- completeClassDefinition(Class, value)
+      }
     value
 }
 
@@ -147,7 +200,7 @@ checkSlotAssignment <- function(obj, name, value)
     if(identical(slot, class(value)))
        return(value)
     if(is(value, slot))
-       return(as(value, slot))
+       return(as(value, slot, strict=FALSE))
     else
        stop(paste("Value supplied is not valid for slot \"", name, "\", is(value, \"", slot,
                    "\") is not TRUE", sep=""))
@@ -176,22 +229,49 @@ slotNames <-
     if(is.character(x) && length(x) == 1)
         Class <- x
     else
-        Class <- data.class(x)
+        Class <- .class1(x)
     names(getProperties(getClass(Class)))
 }
 
 removeClass <-
   ## Remove the definition of this class.
-  function(Class, where = -1)
+  function(Class, where = -1, removeSubclassLinks = TRUE)
 {
     ## always remove the cached version, if any
-    Class <- classMetaName(Class)
-    if(!is.null(getFromClassMetaData(Class)))
-        removeFromClassMetaData(Class)
-    if(identical(where, -1))
-        rm(list=Class, pos=.GlobalEnv)
-    else if(!identical(where, 0))
-        rm(list=Class, pos=where)
+    what <- classMetaName(Class)
+    if(removeSubclassLinks && !identical(where, 0)) {
+        fullDef <- trySilent(getClass(Class))
+        if(is(fullDef, "try-error")) {
+            warning("unable to get definition of \"", Class, "\" (",
+                    fullDef, ")")
+            fullDef <- NULL
+        }
+    }
+    else
+        fullDef <- getFromClassMetaData(what)
+    if(!is.null(fullDef))
+        removeFromClassMetaData(what)
+    if(identical(where, 0))
+    {}
+    else {
+        ClassDef <- getClassDef(Class)
+        if(is.null(ClassDef)) {
+            warning("\"", Class, "\" is not a class (no action taken)")
+            return(FALSE)
+        }
+        if(identical(where, -1))
+            rm(list=what, pos=find(what)[[1]])
+        else
+            rm(list=what, pos=where)
+        ## remove extensions
+        what <- extendsMetaName(ClassDef)
+        extWhere <- find(what)
+        for(pos in extWhere)
+            rm(list = what, pos = pos)
+        if(removeSubclassLinks && is(fullDef, "classRepresentation"))
+            .removeSubclassLinks(ClassDef@className, classDef@package, fullDef@subclasses)
+    }
+    TRUE
 }
 
 
@@ -207,7 +287,7 @@ isClass <-
 new <-
   ## Generate an object from the specified class.
   ##
-  ## If other arguments are included, these are the values for named properties
+  ## If other arguments are included, these are the values for named slots
   ## in the object, or an object that can be coerced into this class.
   ## Note that the basic vector classes, `"numeric"', etc. are implicitly defined,
   ## so one can use `new' for these classes.
@@ -269,9 +349,9 @@ validObject <- function(object, test = FALSE) {
   extends <- names(extendType); i <- 1
   while(length(errors) == 0 && i <= length(extends)) {
     superClass <- extends[[i]]
-    test <- extendType[[i]]$test
+    testFun <- extendType[[i]]$test
     i <- i+1
-    if(is.function(test) && !is(object, superClass))
+    if(is.function(testFun) && !is(object, superClass))
       next ## skip conditional relations that don't hold for this object
     validityMethod <- getValidity(getClassDef(superClass))
     if(is(validityMethod, "function"))
@@ -296,35 +376,48 @@ validObject <- function(object, test = FALSE) {
 }
 
 setValidity <-
-  function(ClassDef, method) {
-    if(!isClassDef(ClassDef))
-      ClassDef <- getClassDef(ClassDef)
+  function(Class, method, where = 1) {
+    if(isClassDef(Class)) {
+        ClassDef <- Class
+        Class <- ClassDef@className
+    }
+    else {
+      ClassDef <- getClassDef(Class)
+  }
+    method <- .makeValidityMethod(Class, method)
     if(is.null(method) ||
       (is(method, "function") && length(formalArgs(method))==1))
-      setInClassDef(ClassDef, ".Validity", method)
+      ClassDef@validity <- method
     else
       stop("validity method must be NULL or a function of one argument")
+    ## TO DO:  check the where argument against the package of the class def.
+    assignClassDef(Class, ClassDef, where = where)
     resetClass(ClassDef)
   }
 
 resetClass <-
-    function(Class) {
-        if(isClassDef(Class))
+    function(Class, 
+             resetSubclasses = TRUE) {
+        if(is(Class, "classRepresentation"))
             Class <- getClassName(Class)
-        cname <- classMetaName(Class)
+        cname <- classMetaName(Class) ## TODO:  change to allow same name, different package
         def <- getFromClassMetaData(cname)
-        if(!is.null(def))
+        if(!is.null(def)) {
             removeFromClassMetaData(cname)
+            if(resetSubclasses)
+                for(what in names(def@subclasses))
+                    resetClass(what) # no further recursion needed if def is complete
+        }
         Class
     }
 
 ## the (default) initialization:  becomes the default method when the function
 ## is made a generic by .InitMethodDefinitions
 
-initialize <- function(object, ...) {
+initialize <- function(.Object, ...) {
     args <- list(...)
     if(length(args) > 0) {
-        Class <- class(object)
+        Class <- class(.Object)
         ## the basic classes have fixed definitions
         if(!is.na(match(Class, .BasicClasses)))
             return(newBasic(Class, ...))
@@ -335,17 +428,21 @@ initialize <- function(object, ...) {
         elements <- args[which]
         supers <- args[!which]
         thisExtends <- names(getExtends(ClassDef))
+        slotDefs <- getProperties(ClassDef)
+        dataPart <- elNamed(slotDefs, ".Data")
+        if(is.null(dataPart)) dataPart <- "missing"
         if(length(supers) > 0) {
             for(i in rev(seq(along = supers))) {
                 obj <- el(supers, i)
-                Classi <- data.class(obj)
+                Classi <- .class1(obj)
                 if(identical(Classi, Class))
-                    object <- obj
+                    .Object <- obj
                 else if(extends(Classi, Class))
-                    ## typically, throw away extra slots.
-                    object <- as(obj, Class)
+                    .Object <- as(obj, Class, strict=FALSE)
+                else if(extends(Classi, dataPart))
+                    .Object@.Data <- obj
                 else if(extends(Class, Classi))
-                    as(object, Classi) <- obj
+                    as(.Object, Classi) <- obj
                 else {
                     ## is there a class to which we can coerce obj
                     ## that is then among the superclasses of Class?
@@ -356,7 +453,7 @@ initialize <- function(object, ...) {
                     which <- seq(along=which)[!is.na(which)]
                     if(length(which) >0 ) {
                         Classi <- thisExtends[which[1]]
-                        as(object, Classi) <- as(obj, Classi)
+                        as(.Object, Classi) <- as(obj, Classi, strict = FALSE)
                     }
                     else
                         stop(paste("Can't use object of class \"", Classi,
@@ -365,14 +462,13 @@ initialize <- function(object, ...) {
             }
         }
         if(length(elements)>0) {
-            slotDefs <- getProperties(ClassDef)
             snames <- names(elements)
             if(any(duplicated(snames)))
                 stop(paste("Duplicated slot names:",
                            paste(snames[duplicated(snames)], collapse = ", ")), sep="")
             which  <- match(snames, names(slotDefs))
             if(any(is.na(which)))
-                stop(paste("Invalid names for properties of class ",
+                stop(paste("Invalid names for slots of class ",
                            Class, ": ", paste(snames[is.na(which)], collapse=", "), sep=""))
             for(i in seq(along=snames)) {
                 slotName <- el(snames, i)
@@ -382,10 +478,21 @@ initialize <- function(object, ...) {
                     stop(paste("Invalid object for slot \"", slotName,
                                "\", with class \"", class(slotVal), 
                                "\", should be or extend class \"", slotClass, "\"", sep = ""))
-                slotVal <- as(slotVal, slotClass)
-                slot(object, slotName, check = FALSE) <- slotVal
+                slotVal <- as(slotVal, slotClass, strict = FALSE)
+                slot(.Object, slotName, check = FALSE) <- slotVal
             }
         }
     }
-    object
+    .Object
+}
+
+findClass <- function(Class) {
+    if(!is.character(Class))
+        Class <- getClassName(Class)
+    what <- classMetaName(Class)
+    where <- search()
+    ok <- logical(length(where))
+    for(i in seq(along=where))
+        ok[i] <- exists(what, where[i], inherits = FALSE)
+    where[ok]
 }
