@@ -1,4 +1,4 @@
-### $Id: zzModels.R,v 1.5 2001/05/10 02:50:31 bates Exp $
+### $Id: zzModels.R,v 1.6 2002/06/14 20:22:57 ripley Exp $
 ###
 ###       Individual selfStarting nonlinear regression models
 ###
@@ -501,7 +501,7 @@ SSweibull <- # selfStart( ~ Asym - Drop * exp(-exp(lrc)*x^pwr),
                   stop("Too few distinct input values to fit the Weibull growth model")
               }
               if (any(xy[["x"]] < 0)) {
-                  error("All x values must be non-negative to fit the Weibull growth model")
+                  stop("All x values must be non-negative to fit the Weibull growth model")
               }
               Rasym <- NLSstRtAsymptote(xy)
               Lasym <- NLSstLfAsymptote(xy)
