@@ -208,8 +208,8 @@ ccf <- function(x, y, lag.max = NULL,
     X <- na.action(ts.union(x, y))
     colnames(X) <- c(deparse(substitute(x)), deparse(substitute(y)))
     acf.out <- acf(X, lag.max = lag.max, plot = FALSE, type = type)
-    lag <- c(rev(acf.out$lag[-1,2,1]), 0, acf.out$lag[,1,2])
-    y   <- c(rev(acf.out$acf[-1,2,1]), 0, acf.out$acf[,1,2])
+    lag <- c(rev(acf.out$lag[-1,2,1]), acf.out$lag[,1,2])
+    y   <- c(rev(acf.out$acf[-1,2,1]), acf.out$acf[,1,2])
     acf.out$acf <- array(y, dim=c(length(y),1,1))
     acf.out$lag <- array(lag, dim=c(length(y),1,1))
     acf.out$snames <- paste(acf.out$snames, collapse = " & ")
