@@ -361,7 +361,15 @@ static SEXP lang2str(SEXP obj, SEXPTYPE t)
 }
 
 /* the S4-style class: for dispatch  required to be a single string;
-   for the newClass function, keeps S3-style multiple classes. */
+   for the newClass function, keeps S3-style multiple classes.
+
+Future Note:  the primitive dataClass in base calls this.  The methods
+package makes the "class" function be that same primitive. The
+"class<-" implementation for the methods package (in main/coerce.c
+under R_set_class) also calls R_data_class. It's possible, though not
+totally obvious, that all this code could be moved into the methods package.
+
+ */
 
 SEXP R_data_class(SEXP obj, int singleString)
 {
@@ -911,7 +919,14 @@ SEXP GetArrayDimnames(SEXP x)
 
 static SEXP pseudo_NULL = 0;
 
-static void init_pseudo_NULL() {
+static SEXP s_dot_Data;
+static SEXP s_getDataPart;
+static SEXP s_setDataPart;
+
+static void init_slot_handling() {
+    s_dot_Data = install(".Data");
+    s_getDataPart = install("getDataPart");
+    s_setDataPart = install("setDataPart");
     /* create and preserve an object that is NOT R_NilValue, and is used
        to represent slots that are NULL (which an attribute can not
        be).  The point is not just to store NULL as a slot, but also to
@@ -922,57 +937,82 @@ static void init_pseudo_NULL() {
     pseudo_NULL = install("\001NULL\001");
 }
 
+static SEXP data_part(SEXP obj) {
+    SEXP e, val;
+    if(!s_getDataPart)
+	init_slot_handling();
+    PROTECT(e = allocVector(LANGSXP, 2));
+    SETCAR(e, s_getDataPart);
+    val = CDR(e);
+    SETCAR(val, obj);
+    val = eval(e, R_GlobalEnv);
+    UNPROTECT(1);
+    return(val);
+}
+
+static SEXP set_data_part(SEXP obj,  SEXP rhs) {
+    SEXP e, val;
+    if(!s_setDataPart)
+	init_slot_handling();
+    PROTECT(e = allocVector(LANGSXP, 3));
+    SETCAR(e, s_setDataPart);
+    val = CDR(e);
+    SETCAR(val, obj);
+    val = CDR(val);
+    SETCAR(val, rhs);
+    val = eval(e, R_GlobalEnv);
+    UNPROTECT(1);
+    return(val);
+}
+
 SEXP R_do_slot(SEXP obj, SEXP name) {
   /* Slots are stored as attributes to
      provide some back-compatibility
   */
     SEXP value = NULL; int nprotect = 0;
-    SEXP input = name;
-    if(isSymbol(name) ) {
-	input = PROTECT(allocVector(STRSXP, 1));  nprotect++;
-	SET_STRING_ELT(input, 0, PRINTNAME(name));
-    }
-    else if(!(isString(name) && LENGTH(name) == 1))
+    if(!(isSymbol(name) || (isString(name) && LENGTH(name) == 1)))
 	error("invalid type or length for slot name");
-    value = getAttrib(obj, input);
-    if(value == R_NilValue)
-	/* not there.  But since even NULL really does get stored, this
+    if(!s_dot_Data)
+	init_slot_handling();
+    if(isString(name)) name = install(CHAR(STRING_ELT(name, 0)));
+    if(name == s_dot_Data)
+	return data_part(obj);
+    value = getAttrib(obj, name);
+    if(value == R_NilValue) {
+	SEXP input = name;
+	if(isSymbol(name) ) {
+	    input = PROTECT(allocVector(STRSXP, 1));  nprotect++;
+	SET_STRING_ELT(input, 0, PRINTNAME(name));
+	}
+ 	/* not there.  But since even NULL really does get stored, this
 	   implies that there is no slot of this name.  Or somebody
 	   screwed up by using atttr(..) <- NULL */
 	error("\"%s\" is not a valid slot for this object (or was mistakenly deleted)",
 	      CHAR(asChar(input)));
+    }
     else if(value == pseudo_NULL)
 	value = R_NilValue;
     UNPROTECT(nprotect);
     return value;
 }
 
-SEXP R_do_slot_assign(SEXP obj, SEXP name, SEXP check, SEXP value) {
+SEXP R_do_slot_assign(SEXP obj, SEXP name, SEXP value) {
     SEXP input = name; int nprotect = 0;
-    Rboolean do_check = (R_has_methods(NULL) && 
-			 (check == R_NilValue || 
-			  LOGICAL_VALUE(check)));
-    if(do_check)
-	/* call the S language checker:  it will generate an error if
-	   the check fails, and return a possibly coerced value otherwise */
-	value = R_do_slot_check(obj, name, value);
     if(isSymbol(name) ) {
 	input = PROTECT(allocVector(STRSXP, 1)); nprotect++;
 	SET_STRING_ELT(input, 0, PRINTNAME(name));
     }
     else if(!(isString(name) && LENGTH(name) == 1))
 	error("invalid type or length for slot name");
-    if(do_check)
-	/* check that the slot is currently there. Will generate an
-	   error if the name is a non-slot. */
-	R_do_slot(obj, input);
-    if(value == R_NilValue) {
+    if(!s_dot_Data)
+	init_slot_handling();
+    if(isString(name)) name = install(CHAR(STRING_ELT(name, 0)));
+    if(name == s_dot_Data)
+	return set_data_part(obj, value);
+    if(isNull(value))
 	/* slots, but not attributes, can be NULL.  Store a special symbol
 	   instead. */
-	if(pseudo_NULL == 0)
-	    init_pseudo_NULL();
 	value = pseudo_NULL;
-    }
     PROTECT(obj); nprotect++;
     setAttrib(obj, input, value);
     UNPROTECT(nprotect);
@@ -981,7 +1021,7 @@ SEXP R_do_slot_assign(SEXP obj, SEXP name, SEXP check, SEXP value) {
 
 SEXP R_pseudo_null() {
     if(pseudo_NULL == 0)
-	init_pseudo_NULL();
+	init_slot_handling();
     return pseudo_NULL;
 }
 
@@ -1012,8 +1052,7 @@ SEXP do_AT_assign(SEXP call, SEXP op, SEXP args, SEXP env)
        this is not quite right.  It can, at the least, be a promise
        for the "@" case. */
     value = eval(CADDR(args), env);
-    ans = R_do_slot_assign(object, nlist, R_NilValue/* force check */
-			   , value);
+    ans = R_do_slot_assign(object, nlist, value);
     UNPROTECT(1);
     return ans;
 }
