@@ -1,22 +1,24 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2000  Robert Gentleman, Ross Ihaka and the
- *                            R Development Core Team
+ *  Copyright (C) 1997--2002  Robert Gentleman, Ross Ihaka and the
+ *			      R Development Core Team
+ *  Copyright (C) 2002	      The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  the Free Software Foundation; either version 2, or (at your option)
+ *  any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  A copy of the GNU General Public License is available via WWW at
+ *  http://www.gnu.org/copyleft/gpl.html.  You can also obtain it by
+ *  writing to the Free Software Foundation, Inc., 59 Temple Place,
+ *  Suite 330, Boston, MA  02111-1307  USA.
  */
 
 /* Code to handle list / vector switch */
@@ -311,15 +313,15 @@ static void ComplexAnswer(SEXP x, struct BindData *data)
 	n = LENGTH(x);
 	for (i = 0; i < n; i++) {
 	    xi = INTEGER(x)[i];
-            if (xi == NA_INTEGER) {
-                COMPLEX(data->ans_ptr)[data->ans_length].r = NA_REAL;
-                COMPLEX(data->ans_ptr)[data->ans_length].i = NA_REAL;
-            }
-            else {
-                COMPLEX(data->ans_ptr)[data->ans_length].r = xi;
-                COMPLEX(data->ans_ptr)[data->ans_length].i = 0.0;
-            }
-            data->ans_length++;
+	    if (xi == NA_INTEGER) {
+		COMPLEX(data->ans_ptr)[data->ans_length].r = NA_REAL;
+		COMPLEX(data->ans_ptr)[data->ans_length].i = NA_REAL;
+	    }
+	    else {
+		COMPLEX(data->ans_ptr)[data->ans_length].r = xi;
+		COMPLEX(data->ans_ptr)[data->ans_length].i = 0.0;
+	    }
+	    data->ans_length++;
 	}
 	break;
     }
@@ -395,11 +397,11 @@ struct NameData {
  int count;
  int seqno;
  int firstpos;
-}; 
+};
 
 
-static void NewExtractNames(SEXP v, SEXP base, SEXP tag, int recurse, 
-                             struct BindData *data, struct NameData *nameData)
+static void NewExtractNames(SEXP v, SEXP base, SEXP tag, int recurse,
+			     struct BindData *data, struct NameData *nameData)
 {
     SEXP names, namei;
     int i, n, savecount=0, saveseqno, savefirstpos=0;
@@ -507,18 +509,18 @@ static SEXP ExtractOptionals(SEXP ans, int *recurse, int *usenames)
 		*recurse = v;
 	    }
 	    if (last == NULL)
-	        last = ans = next;
+		last = ans = next;
 	    else
-	        SETCDR(last, next);
+		SETCDR(last, next);
 	}
 	else if (n != R_NilValue &&  pmatch(R_UseNamesSymbol, n, 1)) {
 	    if ((v = asLogical(CAR(a))) != NA_INTEGER) {
 		*usenames = v;
 	    }
 	    if (last == NULL)
-	        last = ans = next;
+		last = ans = next;
 	    else
-	        SETCDR(last, next);
+		SETCDR(last, next);
 	}
 	else last = a;
     }
@@ -592,13 +594,13 @@ SEXP do_c_dflt(SEXP call, SEXP op, SEXP args, SEXP env)
     /* we use the natural coercion for vector types. */
 
     mode = NILSXP;
-    if (data.ans_flags & 256)      mode = EXPRSXP;
+    if (data.ans_flags & 256)	   mode = EXPRSXP;
     else if (data.ans_flags & 128) mode = VECSXP;
     else if (data.ans_flags &  64) mode = STRSXP;
     else if (data.ans_flags &  32) mode = CPLXSXP;
     else if (data.ans_flags &  16) mode = REALSXP;
-    else if (data.ans_flags &   8) mode = INTSXP;
-    else if (data.ans_flags &   1) mode = LGLSXP;
+    else if (data.ans_flags &	8) mode = INTSXP;
+    else if (data.ans_flags &	1) mode = LGLSXP;
 
     /* Allocate the return value and set up to pass through */
     /* the arguments filling in values of the returned object. */
@@ -737,8 +739,8 @@ SEXP do_unlist(SEXP call, SEXP op, SEXP args, SEXP env)
     else if (data.ans_flags &  64) mode = STRSXP;
     else if (data.ans_flags &  32) mode = CPLXSXP;
     else if (data.ans_flags &  16) mode = REALSXP;
-    else if (data.ans_flags &   8) mode = INTSXP;
-    else if (data.ans_flags &   1) mode = LGLSXP;
+    else if (data.ans_flags &	8) mode = INTSXP;
+    else if (data.ans_flags &	1) mode = LGLSXP;
 
     /* Allocate the return value and set up to pass through */
     /* the arguments filling in values of the returned object. */
@@ -841,14 +843,14 @@ SEXP FetchMethod(char *generic, char *classname, SEXP env)
     return method;
 }
 
+/* cbind(...) and rbind(...) : */
 SEXP do_bind(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    SEXP a, t, obj, class, classlist, classname, method, classmethod;
+    SEXP a, t, obj, class, classlist, classname, method, classmethod, rho;
     char *generic;
-    int mode = ANYSXP;
+    int mode;
     struct BindData data;
-    SEXP rho;
-      data.deparse_level = 1;
+    data.deparse_level = 1;
 
     /* Lazy evaluation and method dispatch based on argument types are
      * fundamentally incompatible notions.  The results here are
@@ -862,19 +864,18 @@ SEXP do_bind(SEXP call, SEXP op, SEXP args, SEXP env)
      * The dispatch rule here is as follows:
      *
      * 1) For each argument we get the list of possible class
-     *    memberships from the class attribute.
+     *	  memberships from the class attribute.
      *
      * 2) We inspect each class in turn to see if there is an
-     *    an applicable method.
+     *	  an applicable method.
      *
      * 3) If we find an applicable method we make sure that it is
-     *    identical to any method determined for prior arguments.
-     *    If it is identical, we proceed, otherwise we immediately
-     *    drop through to the default code.
+     *	  identical to any method determined for prior arguments.
+     *	  If it is identical, we proceed, otherwise we immediately
+     *	  drop through to the default code.
      */
 
     PROTECT(args = promiseArgs(args, env));
-    mode = 0;
     generic = (PRIMVAL(op) == 1) ? "cbind" : "rbind";
     class = R_NilValue;
     method = R_NilValue;
@@ -919,21 +920,23 @@ SEXP do_bind(SEXP call, SEXP op, SEXP args, SEXP env)
     /* First, extract the evaluated arguments. */
 #ifdef OLD
     for (a = args; a != R_NilValue; a = CDR(a))
-        CAR(a) = PRVALUE(CAR(a));
+	CAR(a) = PRVALUE(CAR(a));
 #endif
 
-    rho = env; /* GLOBAL */
+    rho = env;
     data.ans_flags = 0;
     data.ans_length = 0;
     data.ans_nnames = 0;
-
     for (t = args; t != R_NilValue; t = CDR(t))
 	AnswerType(PRVALUE(CAR(t)), 0, 0, &data);
 
-    /* zero-extent matrices shouldn't give NULL, anymore :
-       if (ans_length == 0)
-       return R_NilValue;
-    */
+    /* zero-extent matrices shouldn't give NULL, but cbind(NULL) should: */
+    if (!data.ans_flags && !data.ans_length) {
+	UNPROTECT(1);
+	return R_NilValue;
+    }
+
+    mode = NILSXP;
     if (data.ans_flags >= 128) {
 	if (data.ans_flags & 128)
 	    mode = LISTSXP;
@@ -992,7 +995,7 @@ static SEXP cbind(SEXP call, SEXP args, SEXPTYPE mode, SEXP rho)
     int i, j, k, idx, n;
     int have_rnames, have_cnames;
     int nnames, mnames;
-    int rows, cols, mrows;
+    int rows, cols, mrows, lenmin = 0;
     int warned;
     SEXP dn, t, u, result, dims, expr;
     struct BindData data;
@@ -1004,27 +1007,32 @@ static SEXP cbind(SEXP call, SEXP args, SEXPTYPE mode, SEXP rho)
     mnames = 0;
     rows = 0;
     cols = 0;
-    /* mrows = 0;*/
     mrows = -1;
 
+    /* check if we are in the zero-row case */
+
+    for (t = args; t != R_NilValue; t = CDR(t))
+	if(length(PRVALUE(CAR(t))) > 0) {
+	    lenmin = 1;
+	    break;
+	}
+
     /* check conformability of matrix arguments */
 
     n = 0;
     for (t = args; t != R_NilValue; t = CDR(t)) {
 	u = PRVALUE(CAR(t));
-	if (length(u) >= 0) {
-	    dims = getAttrib(u, R_DimSymbol);
-	    if (length(dims) == 2) {
-		if (mrows == -1)
-		    mrows = INTEGER(dims)[0];
-		else if (mrows != INTEGER(dims)[0])
-		    errorcall(call, "number of rows of matrices must match (see arg %d)", n + 1);
-		cols += INTEGER(dims)[1];
-	    }
-	    else if (length(u)>0) {
-		rows = imax2(rows, length(u));
-		cols += 1;
-	    }
+	dims = getAttrib(u, R_DimSymbol);
+	if (length(dims) == 2) {
+	    if (mrows == -1)
+		mrows = INTEGER(dims)[0];
+	    else if (mrows != INTEGER(dims)[0])
+		errorcall(call, "number of rows of matrices must match (see arg %d)", n + 1);
+	    cols += INTEGER(dims)[1];
+	}
+	else if (length(u) >= lenmin) {
+	    rows = imax2(rows, length(u));
+	    cols += 1;
 	}
 	n++;
     }
@@ -1037,33 +1045,31 @@ static SEXP cbind(SEXP call, SEXP args, SEXPTYPE mode, SEXP rho)
     for (t = args; t != R_NilValue; t = CDR(t)) {
 	u = PRVALUE(CAR(t));
 	n++;
-	if (length(u) >= 0) {
-	    dims = getAttrib(u, R_DimSymbol);
-	    if (length(dims) == 2) {
-		dn = getAttrib(u, R_DimNamesSymbol);
-		if (length(dn) == 2) {
-		    if (VECTOR_ELT(dn, 1) != R_NilValue)
-			have_cnames = 1;
-		    if (VECTOR_ELT(dn, 0) != R_NilValue)
-			mnames = mrows;
-		}
-	    }
-	    else {
-		k = length(u);
-		if (!warned && k>0 && (k > rows || rows % k)) {
-		    warned = 1;
-		    PROTECT(call = substituteList(call, rho));
-		    warningcall(call, "number of rows of result\n\tis not a multiple of vector length (arg %d)", n);
-		    UNPROTECT(1);
-		}
-		dn = getAttrib(u, R_NamesSymbol);
-		if (k && (TAG(t) != R_NilValue || 
-		    ((data.deparse_level == 1) && 
-		     isSymbol(substitute(CAR(t),R_NilValue)))))
+	dims = getAttrib(u, R_DimSymbol);
+	if (length(dims) == 2) {
+	    dn = getAttrib(u, R_DimNamesSymbol);
+	    if (length(dn) == 2) {
+		if (VECTOR_ELT(dn, 1) != R_NilValue)
 		    have_cnames = 1;
-		nnames = imax2(nnames, length(dn));
+		if (VECTOR_ELT(dn, 0) != R_NilValue)
+		    mnames = mrows;
 	    }
 	}
+	else {
+	    k = length(u);
+	    if (!warned && k>0 && (k > rows || rows % k)) {
+		warned = 1;
+		PROTECT(call = substituteList(call, rho));
+		warningcall(call, "number of rows of result\n\tis not a multiple of vector length (arg %d)", n);
+		UNPROTECT(1);
+	    }
+	    dn = getAttrib(u, R_NamesSymbol);
+	    if (k >= lenmin && (TAG(t) != R_NilValue ||
+		      ((data.deparse_level == 1) &&
+		       isSymbol(substitute(CAR(t),R_NilValue)))))
+		have_cnames = 1;
+	    nnames = imax2(nnames, length(dn));
+	}
     }
     if (mnames || nnames == rows)
 	have_rnames = 1;
@@ -1074,7 +1080,7 @@ static SEXP cbind(SEXP call, SEXP args, SEXPTYPE mode, SEXP rho)
     if (mode == STRSXP) {
 	for (t = args; t != R_NilValue; t = CDR(t)) {
 	    u = PRVALUE(CAR(t));
-	    if (length(u) > 0) {/* rbind() has ">=" here */
+	    if (length(u) >= lenmin) {
 		u = coerceVector(u, STRSXP);
 		k = LENGTH(u);
 		idx = (!isMatrix(u)) ? rows : k;
@@ -1086,7 +1092,7 @@ static SEXP cbind(SEXP call, SEXP args, SEXPTYPE mode, SEXP rho)
     else if (mode == CPLXSXP) {
 	for (t = args; t != R_NilValue; t = CDR(t)) {
 	    u = PRVALUE(CAR(t));
-	    if (length(u) > 0) {
+	    if (length(u) >= lenmin) {
 		u = coerceVector(u, CPLXSXP);
 		k = LENGTH(u);
 		idx = (!isMatrix(u)) ? rows : k;
@@ -1098,7 +1104,7 @@ static SEXP cbind(SEXP call, SEXP args, SEXPTYPE mode, SEXP rho)
     else {
 	for (t = args; t != R_NilValue; t = CDR(t)) {
 	    u = PRVALUE(CAR(t));
-	    if (length(u) > 0) {
+	    if (length(u) >= lenmin) {
 		k = LENGTH(u);
 		idx = (!isMatrix(u)) ? rows : k;
 		if (TYPEOF(u) <= INTSXP) {
@@ -1130,46 +1136,43 @@ static SEXP cbind(SEXP call, SEXP args, SEXPTYPE mode, SEXP rho)
 	j = 0;
 	for (t = args; t != R_NilValue; t = CDR(t)) {
 	    u = PRVALUE(CAR(t));
-	    if (length(u) >= 0) {
+	    if (isMatrix(u)) {
 
-		if (isMatrix(u)) {
+		v = getAttrib(u, R_DimNamesSymbol);
+		tnam = GetColNames(v);
 
-		    v = getAttrib(u, R_DimNamesSymbol);
-		    tnam = GetColNames(v);
+		if (have_rnames &&
+		    GetRowNames(dn) == R_NilValue &&
+		    GetRowNames(v) != R_NilValue)
+		    SetRowNames(dn, duplicate(GetRowNames(v)));
 
-		    if (have_rnames &&
-			GetRowNames(dn) == R_NilValue &&
-			GetRowNames(v) != R_NilValue)
-			SetRowNames(dn, duplicate(GetRowNames(v)));
-
-		    /* rbind() does this only  if(have_?names) .. : */
-		    if (tnam != R_NilValue) {
-			for (i = 0; i < length(tnam); i++)
-			    SET_STRING_ELT(nam, j++, STRING_ELT(tnam, i));
-		    }
-		    else if (have_cnames) {
-			for (i = 0; i < ncols(u); i++)
-			    SET_STRING_ELT(nam, j++, R_BlankString);
-		    }
+		/* rbind() does this only  if(have_?names) .. : */
+		if (tnam != R_NilValue) {
+		    for (i = 0; i < length(tnam); i++)
+			SET_STRING_ELT(nam, j++, STRING_ELT(tnam, i));
 		}
-		else if (length(u) > 0) {
-
-		    u = getAttrib(u, R_NamesSymbol);
-		    tnam = u;
-
-		    if (have_rnames && GetRowNames(dn) == R_NilValue
-			&& u != R_NilValue && length(u) == rows)
-			SetRowNames(dn, duplicate(u));
-
-		    if (TAG(t) != R_NilValue) 
-			SET_STRING_ELT(nam, j++, PRINTNAME(TAG(t)));
-		    else if ((data.deparse_level == 1) && 
-			     isSymbol(expr = substitute(CAR(t), R_NilValue)))
-			SET_STRING_ELT(nam, j++, PRINTNAME(expr));
-		    else if (have_cnames) 
+		else if (have_cnames) {
+		    for (i = 0; i < ncols(u); i++)
 			SET_STRING_ELT(nam, j++, R_BlankString);
 		}
 	    }
+	    else if (length(u) >= lenmin) {
+
+		u = getAttrib(u, R_NamesSymbol);
+		tnam = u;
+
+		if (have_rnames && GetRowNames(dn) == R_NilValue
+		    && u != R_NilValue && length(u) == rows)
+		    SetRowNames(dn, duplicate(u));
+
+		if (TAG(t) != R_NilValue)
+		    SET_STRING_ELT(nam, j++, PRINTNAME(TAG(t)));
+		else if ((data.deparse_level == 1) &&
+			 isSymbol(expr = substitute(CAR(t), R_NilValue)))
+		    SET_STRING_ELT(nam, j++, PRINTNAME(expr));
+		else if (have_cnames)
+		    SET_STRING_ELT(nam, j++, R_BlankString);
+	    }
 	}
 	setAttrib(result, R_DimNamesSymbol, dn);
 	UNPROTECT(1);
@@ -1178,12 +1181,13 @@ static SEXP cbind(SEXP call, SEXP args, SEXPTYPE mode, SEXP rho)
     return result;
 } /* cbind */
 
+
 static SEXP rbind(SEXP call, SEXP args, SEXPTYPE mode, SEXP rho)
 {
     int i, j, k, n;
     int have_rnames, have_cnames;
     int nnames, mnames;
-    int rows, cols, mcols, mrows,have_mcols;
+    int rows, cols, mcols, mrows, have_mcols, lenmin = 0;
     int warned;
     SEXP dn, t, u, result, dims, expr;
     struct BindData data;
@@ -1198,33 +1202,38 @@ static SEXP rbind(SEXP call, SEXP args, SEXPTYPE mode, SEXP rho)
     mcols = 0;
     have_mcols=0;
 
+    /* check if we are in the zero-cols case */
+
+    for (t = args; t != R_NilValue; t = CDR(t))
+	if(length(PRVALUE(CAR(t))) > 0) {
+	    lenmin = 1;
+	    break;
+	}
     /* check conformability of matrix arguments */
 
     n = 0;
     for (t = args; t != R_NilValue; t = CDR(t)) {
 	u = PRVALUE(CAR(t));
-	if (length(u) >= 0) {
-	    dims = getAttrib(u, R_DimSymbol);
-	    if (length(dims) == 2) {
-		if (!have_mcols){
-		    mcols = INTEGER(dims)[1];
-		    have_mcols=1;
-		}
-		else if (mcols != INTEGER(dims)[1])
-		    errorcall(call, "number of columns of matrices must match (see arg %d)", n + 1);
-		rows += INTEGER(dims)[0];
-	    }
-	    else if (length(u)>0){
-		cols = imax2(cols, length(u));
-		rows += 1;
+	dims = getAttrib(u, R_DimSymbol);
+	if (length(dims) == 2) {
+	    if (!have_mcols){
+		mcols = INTEGER(dims)[1];
+		have_mcols=1;
 	    }
+	    else if (mcols != INTEGER(dims)[1])
+		errorcall(call, "number of columns of matrices must match (see arg %d)", n + 1);
+	    rows += INTEGER(dims)[0];
+	}
+	else if (length(u) >= lenmin){
+	    cols = imax2(cols, length(u));
+	    rows += 1;
 	}
 	n++;
     }
-    if (have_mcols) 
-	    cols = mcols;
+    if (have_mcols)
+	cols = mcols;
+
 
-	    
 
     /* Check conformability of vector arguments. -- Look for dimnames. */
 
@@ -1233,33 +1242,31 @@ static SEXP rbind(SEXP call, SEXP args, SEXPTYPE mode, SEXP rho)
     for (t = args; t != R_NilValue; t = CDR(t)) {
 	u = PRVALUE(CAR(t));
 	n++;
-	if (length(u) >= 0) {
-	    dims = getAttrib(u, R_DimSymbol);
-	    if (length(dims) == 2) {
-		dn = getAttrib(u, R_DimNamesSymbol);
-		if (length(dn) == 2) {
-		    if (VECTOR_ELT(dn, 0) != R_NilValue)
-			have_rnames = 1;
-		    if (VECTOR_ELT(dn, 1) != R_NilValue)
-			mnames = mcols;
-		}
-	    }
-	    else {
-		k = length(u);
-		if (!warned && k>0 && (k > cols || cols % k)) {
-		    warned = 1;
-		    PROTECT(call = substituteList(call, rho));
-		    warningcall(call, "number of columns of result\n\tnot a multiple of vector length (arg %d)", n);
-		    UNPROTECT(1);
-		}
-		dn = getAttrib(u, R_NamesSymbol);
-		if (k && (TAG(t) != R_NilValue || 
-		    ((data.deparse_level == 1) && 
-		     isSymbol(substitute(CAR(t),R_NilValue)))))
+	dims = getAttrib(u, R_DimSymbol);
+	if (length(dims) == 2) {
+	    dn = getAttrib(u, R_DimNamesSymbol);
+	    if (length(dn) == 2) {
+		if (VECTOR_ELT(dn, 0) != R_NilValue)
 		    have_rnames = 1;
-		nnames = imax2(nnames, length(dn));
+		if (VECTOR_ELT(dn, 1) != R_NilValue)
+		    mnames = mcols;
 	    }
 	}
+	else {
+	    k = length(u);
+	    if (!warned && k>0 && (k > cols || cols % k)) {
+		warned = 1;
+		PROTECT(call = substituteList(call, rho));
+		warningcall(call, "number of columns of result\n\tnot a multiple of vector length (arg %d)", n);
+		UNPROTECT(1);
+	    }
+	    dn = getAttrib(u, R_NamesSymbol);
+	    if (k >= lenmin && (TAG(t) != R_NilValue ||
+		      ((data.deparse_level == 1) &&
+		       isSymbol(substitute(CAR(t),R_NilValue)))))
+		have_rnames = 1;
+	    nnames = imax2(nnames, length(dn));
+	}
     }
     if (mnames || nnames == cols)
 	have_cnames = 1;
@@ -1270,7 +1277,7 @@ static SEXP rbind(SEXP call, SEXP args, SEXPTYPE mode, SEXP rho)
     if (mode == STRSXP) {
 	for (t = args; t != R_NilValue; t = CDR(t)) {
 	    u = PRVALUE(CAR(t));
-	    if (length(u) > 0) {
+	    if (length(u) >= lenmin) {
 		u = coerceVector(u, STRSXP);
 		k = LENGTH(u);
 		mrows = (isMatrix(u)) ? nrows(u) : 1;
@@ -1283,12 +1290,11 @@ static SEXP rbind(SEXP call, SEXP args, SEXPTYPE mode, SEXP rho)
 		n += mrows;
 	    }
 	}
-	/* 0.63.3 erroneously:	return result; */
     }
     else if (mode == CPLXSXP) {
 	for (t = args; t != R_NilValue; t = CDR(t)) {
 	    u = PRVALUE(CAR(t));
-	    if (length(u) > 0) {
+	    if (length(u) >= lenmin) {
 		u = coerceVector(u, CPLXSXP);
 		k = LENGTH(u);
 		mrows = (isMatrix(u)) ? nrows(u) : 1;
@@ -1299,12 +1305,11 @@ static SEXP rbind(SEXP call, SEXP args, SEXPTYPE mode, SEXP rho)
 		n += mrows;
 	    }
 	}
-	/* 0.63.3 erroneously:	return result; */
     }
     else {
 	for (t = args; t != R_NilValue; t = CDR(t)) {
 	    u = PRVALUE(CAR(t));
-	    if (length(u)>=0) {
+	    if (length(u) >= lenmin) {
 		k = LENGTH(u);
 		/* mrows = (isMatrix(u)) ? nrows(u) : 1; */
 		if (isMatrix(u)) {
@@ -1350,46 +1355,43 @@ static SEXP rbind(SEXP call, SEXP args, SEXPTYPE mode, SEXP rho)
 	j = 0;
 	for (t = args; t != R_NilValue; t = CDR(t)) {
 	    u = PRVALUE(CAR(t));
-	    if (length(u) >= 0) {
+	    if (isMatrix(u)) {
 
-		if (isMatrix(u)) {
+		v = getAttrib(u, R_DimNamesSymbol);
+		tnam = GetRowNames(v);
 
-		    v = getAttrib(u, R_DimNamesSymbol);
-		    tnam = GetRowNames(v);
+		if (have_cnames && GetColNames(dn) == R_NilValue
+		    && GetColNames(v) != R_NilValue)
+		    SetColNames(dn, duplicate(GetColNames(v)));
 
-		    if (have_cnames && GetColNames(dn) == R_NilValue
-			&& GetColNames(v) != R_NilValue)
-			SetColNames(dn, duplicate(GetColNames(v)));
-
-		    /* cbind() doesn't test have_?names BEFORE tnam!=Nil..:*/
-		    if (have_rnames) {
-			if (tnam != R_NilValue) {
-			    for (i = 0; i < length(tnam); i++)
-				SET_STRING_ELT(nam, j++, STRING_ELT(tnam, i));
-			}
-			else {
-			    for (i = 0; i < nrows(u); i++)
+		/* cbind() doesn't test have_?names BEFORE tnam!=Nil..:*/
+		if (have_rnames) {
+		    if (tnam != R_NilValue) {
+			for (i = 0; i < length(tnam); i++)
+			    SET_STRING_ELT(nam, j++, STRING_ELT(tnam, i));
+		    }
+		    else {
+			for (i = 0; i < nrows(u); i++)
 				SET_STRING_ELT(nam, j++, R_BlankString);
-			}
 		    }
 		}
-		else if (length(u) > 0) {
+	    }
+	    else if (length(u) >= lenmin) {
 
-		    u = getAttrib(u, R_NamesSymbol);
-		    tnam = u;
+		u = getAttrib(u, R_NamesSymbol);
+		tnam = u;
 
-		    if (have_cnames && GetColNames(dn) == R_NilValue
-			&& u != R_NilValue && length(u) == cols)
-			SetColNames(dn, duplicate(u));
+		if (have_cnames && GetColNames(dn) == R_NilValue
+		    && u != R_NilValue && length(u) == cols)
+		    SetColNames(dn, duplicate(u));
 
-		    if (TAG(t) != R_NilValue) 
-			SET_STRING_ELT(nam, j++, PRINTNAME(TAG(t)));
-		    else if ((data.deparse_level == 1) && 
-			     isSymbol(expr = substitute(CAR(t), R_NilValue)))
-			SET_STRING_ELT(nam, j++, PRINTNAME(expr));
-		    else if (have_rnames)
-			SET_STRING_ELT(nam, j++, R_BlankString);
-		}
+		if (TAG(t) != R_NilValue)
+		    SET_STRING_ELT(nam, j++, PRINTNAME(TAG(t)));
+		else if ((data.deparse_level == 1) &&
+			 isSymbol(expr = substitute(CAR(t), R_NilValue)))
+		    SET_STRING_ELT(nam, j++, PRINTNAME(expr));
+		else if (have_rnames)
+		    SET_STRING_ELT(nam, j++, R_BlankString);
 	    }
 	}
 	setAttrib(result, R_DimNamesSymbol, dn);
