@@ -32,8 +32,6 @@
 # define hypot pythag
 #endif
 
-static int naflag;
-
 SEXP complex_unary(ARITHOP_TYPE code, SEXP s1)
 {
     int i, n;
@@ -569,9 +567,10 @@ static void z_tanh(Rcomplex *r, Rcomplex *z)
     r->i = -a.r;
 }
 
-static void cmath1(void (*f)(), Rcomplex *x, Rcomplex *y, int n)
+static Rboolean cmath1(void (*f)(), Rcomplex *x, Rcomplex *y, int n)
 {
     int i;
+    Rboolean naflag = FALSE;
     for (i = 0 ; i < n ; i++) {
 	if (ISNA(x[i].r) || ISNA(x[i].i)) {
 	    y[i].r = NA_REAL;
@@ -583,42 +582,44 @@ static void cmath1(void (*f)(), Rcomplex *x, Rcomplex *y, int n)
 	    if(ISNA(y[i].r) || ISNA(y[i].i)) {
 		y[i].r = NA_REAL;
 		y[i].i = NA_REAL;
-		naflag = 1;
+		naflag = TRUE;
 	    }
 #endif
 	}
     }
+
+    return(naflag);
 }
 
 SEXP complex_math1(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP x, y;
     int n;
+    Rboolean naflag = FALSE;
     PROTECT(x = CAR(args));
     n = length(x);
     PROTECT(y = allocVector(CPLXSXP, n));
-    naflag = 0;
 
     switch (PRIMVAL(op)) {
-    case 10002: cmath1(z_atan, COMPLEX(x), COMPLEX(y), n); break;
-    case 10003: cmath1(z_log, COMPLEX(x), COMPLEX(y), n); break;
+    case 10002: naflag = cmath1(z_atan, COMPLEX(x), COMPLEX(y), n); break;
+    case 10003: naflag = cmath1(z_log, COMPLEX(x), COMPLEX(y), n); break;
 
-    case 3:  cmath1(z_sqrt, COMPLEX(x), COMPLEX(y), n); break;
+    case 3: naflag = cmath1(z_sqrt, COMPLEX(x), COMPLEX(y), n); break;
 
-    case 10: cmath1(z_exp, COMPLEX(x), COMPLEX(y), n); break;
+    case 10: naflag = cmath1(z_exp, COMPLEX(x), COMPLEX(y), n); break;
 
-    case 20: cmath1(z_cos, COMPLEX(x), COMPLEX(y), n); break;
-    case 21: cmath1(z_sin, COMPLEX(x), COMPLEX(y), n); break;
-    case 22: cmath1(z_tan, COMPLEX(x), COMPLEX(y), n); break;
-    case 23: cmath1(z_acos, COMPLEX(x), COMPLEX(y), n); break;
-    case 24: cmath1(z_asin, COMPLEX(x), COMPLEX(y), n); break;
+    case 20: naflag = cmath1(z_cos, COMPLEX(x), COMPLEX(y), n); break;
+    case 21: naflag = cmath1(z_sin, COMPLEX(x), COMPLEX(y), n); break;
+    case 22: naflag = cmath1(z_tan, COMPLEX(x), COMPLEX(y), n); break;
+    case 23: naflag = cmath1(z_acos, COMPLEX(x), COMPLEX(y), n); break;
+    case 24: naflag = cmath1(z_asin, COMPLEX(x), COMPLEX(y), n); break;
 
-    case 30: cmath1(z_cosh, COMPLEX(x), COMPLEX(y), n); break;
-    case 31: cmath1(z_sinh, COMPLEX(x), COMPLEX(y), n); break;
-    case 32: cmath1(z_tanh, COMPLEX(x), COMPLEX(y), n); break;
-    case 33: cmath1(z_acosh, COMPLEX(x), COMPLEX(y), n); break;
-    case 34: cmath1(z_asinh, COMPLEX(x), COMPLEX(y), n); break;
-    case 35: cmath1(z_atanh, COMPLEX(x), COMPLEX(y), n); break;
+    case 30: naflag = cmath1(z_cosh, COMPLEX(x), COMPLEX(y), n); break;
+    case 31: naflag = cmath1(z_sinh, COMPLEX(x), COMPLEX(y), n); break;
+    case 32: naflag = cmath1(z_tanh, COMPLEX(x), COMPLEX(y), n); break;
+    case 33: naflag = cmath1(z_acosh, COMPLEX(x), COMPLEX(y), n); break;
+    case 34: naflag = cmath1(z_asinh, COMPLEX(x), COMPLEX(y), n); break;
+    case 35: naflag = cmath1(z_atanh, COMPLEX(x), COMPLEX(y), n); break;
 
 #ifdef NOTYET
 	MATH1(40, lgammafn);
@@ -643,7 +644,7 @@ static SEXP cmath2(SEXP op, SEXP sa, SEXP sb, void (*f)())
     int i, n, na, nb;
     Rcomplex ai, bi, *a, *b, *y;
     SEXP sy;
-
+    int naflag = 0;
     na = length(sa);
     nb = length(sb);
     if ((na == 0) || (nb == 0))
@@ -764,13 +765,17 @@ SEXP do_polyroot(SEXP call, SEXP op, SEXP args, SEXP rho)
 	errorcall(call, "invalid argument type");
     }
     n = length(z);
-    degree = n - 1;
+    degree = 0;
+    for(i = 0; i < n; i++) {
+	if(COMPLEX(z)[i].r!= 0.0 || COMPLEX(z)[i].i != 0.0) degree = i;
+    }
+    n = degree + 1; /* omit trailing zeroes */
     if(degree >= 1) {
 	if(n > 49) errorcall(call, "polynomial degree too high (49 max)");
 	/* <==>	 #define NMAX 50  in  ../appl/cpoly.c */
 
-	if(COMPLEX(z)[n-1].r == 0.0 && COMPLEX(z)[n-1].i == 0.0)
-	    errorcall(call, "highest power has coefficient 0");
+	/* if(COMPLEX(z)[n-1].r == 0.0 && COMPLEX(z)[n-1].i == 0.0)
+	   errorcall(call, "highest power has coefficient 0");*/
 
 	PROTECT(rr = allocVector(REALSXP, n));
 	PROTECT(ri = allocVector(REALSXP, n));
