@@ -56,7 +56,6 @@ SEXP do_readDCF(SEXP call, SEXP op, SEXP args, SEXP env)
     buf = (char *) malloc(buflen);
     if(!buf)
 	error("Could not allocate memory for read.dcf");
-    
     nret = 20;
     /* it is easier if we first have a record per column */
     PROTECT (retval = allocMatrixNA(STRSXP, LENGTH(what), nret));
@@ -65,7 +64,7 @@ SEXP do_readDCF(SEXP call, SEXP op, SEXP args, SEXP env)
     regcomp(&trailblank, "[[:blank:]]+$", REG_EXTENDED);
     regcomp(&contline, "^[[:blank:]]+", REG_EXTENDED);
     regcomp(&regline, "^[^:]+:[[:blank:]]*", REG_EXTENDED);
-    
+
     k=0;
     lastm=-1;
     skip=1;
@@ -89,7 +88,7 @@ SEXP do_readDCF(SEXP call, SEXP op, SEXP args, SEXP env)
 	    if(regexec(&trailblank, line, 1, regmatch, 0)==0){
 		line[regmatch[0].rm_so] = '\0';
 	    }
-	    
+
 	    if(lastm>=0 && regexec(&contline, line, 1, regmatch,
 				   0)==0){
 		need = strlen(line+regmatch[0].rm_eo) +
@@ -168,11 +167,15 @@ SEXP do_readDCF(SEXP call, SEXP op, SEXP args, SEXP env)
     if(!wasopen) con->close(con);
     free(line);
     free(buf);
+    regfree(&blankline);
+    regfree(&contline);
+    regfree(&trailblank);
+    regfree(&regline);
 
     if(skip==0) k++;
 
     /* and now transpose the whole matrix */
-    PROTECT(retval2 = allocMatrixNA(STRSXP, k, LENGTH(what))); 
+    PROTECT(retval2 = allocMatrixNA(STRSXP, k, LENGTH(what)));
     copyMatrix(retval2, retval, 1);
 
     PROTECT(dimnames = allocVector(VECSXP, 2));
