@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Langage for Statistical Data Analysis
  *  Copyright (C) 1995  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997-2001  The R Development Core Team
+ *  Copyright (C) 1997-2002  The R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -52,7 +52,8 @@ static char DLLname[PATH_MAX];
    than vectors and lists unaltered.
 */
 
-static void *RObjToCPtr(SEXP s, int naok, int dup, int narg, int Fort, const char *name, R_toCConverter **converter)
+static void *RObjToCPtr(SEXP s, int naok, int dup, int narg, int Fort, const char *name, R_toCConverter **converter,
+                          int targetType)
 {
     int *iptr;
     float *sptr;
@@ -81,6 +82,16 @@ static void *RObjToCPtr(SEXP s, int naok, int dup, int narg, int Fort, const cha
 	    return(ans);
     }
 
+    if(targetType > 0 && targetType != TYPEOF(s)) {
+     if(!dup) {
+       error("expliciti request not to duplicate arguments in call to %s, but argument %d is of the wrong type", 
+	     name, narg + 1);
+     } 
+
+     if(targetType != SINGLESXP) 
+        s = coerceVector(s, targetType);
+    }
+
     switch(TYPEOF(s)) {
     case LGLSXP:
     case INTSXP:
@@ -181,7 +192,7 @@ static void *RObjToCPtr(SEXP s, int naok, int dup, int narg, int Fort, const cha
 }
 
 
-static SEXP CPtrToRObj(void *p, SEXP arg, int Fort)
+static SEXP CPtrToRObj(void *p, SEXP arg, int Fort, R_NativePrimitiveArgType type)
 {
     int *iptr, n=length(arg);
     float *sptr;
@@ -191,20 +202,19 @@ static SEXP CPtrToRObj(void *p, SEXP arg, int Fort)
     SEXP *lptr, CSingSymbol = install("Csingle");
     int i;
     SEXP s, t;
-    SEXPTYPE type =TYPEOF(arg);
 
     switch(type) {
     case LGLSXP:
     case INTSXP:
 	s = allocVector(type, n);
 	iptr = (int*)p;
-	for(i=0 ; i<n ; i++) {
-	    INTEGER(s)[i] = iptr[i];
-	}
+	for(i=0 ; i<n ; i++) 
+            INTEGER(s)[i] = iptr[i];
 	break;
     case REALSXP:
-	s = allocVector(type, n);
-	if(asLogical(getAttrib(arg, CSingSymbol)) == 1) {
+    case SINGLESXP:
+	s = allocVector(REALSXP, n);
+	if(type == SINGLESXP || asLogical(getAttrib(arg, CSingSymbol)) == 1) {
 	    sptr = (float*) p;
 	    for(i=0 ; i<n ; i++) REAL(s)[i] = (double) sptr[i];
 	} else {
@@ -258,6 +268,20 @@ static SEXP CPtrToRObj(void *p, SEXP arg, int Fort)
     return s;
 }
 
+#ifdef THROW_REGISTRATION_TYPE_ERROR
+static Rboolean
+comparePrimitiveTypes(R_NativePrimitiveArgType type, SEXP s, Rboolean dup)
+{
+   if(TYPEOF(s) == type)
+      return(TRUE);
+
+   if(dup && type == SINGLESXP)
+      return(asLogical(getAttrib(s, install("Csingle"))) == TRUE);
+
+   return(FALSE);
+}
+#endif /* end of THROW_REGISTRATION_TYPE_ERROR */
+
 
 /* Foreign Function Interface.  This code allows a user to call C */
 /* or Fortran code which is either statically or dynamically linked. */
@@ -301,7 +325,8 @@ static SEXP naokfind(SEXP args, int * len, int *naok, int *dup)
 {
     SEXP s, prev;
     int nargs=0, naokused=0, dupused=0, pkgused=0;
-
+    char *p;
+    
     *naok = 0;
     *dup = 1;
     *len = 0;
@@ -315,35 +340,42 @@ static SEXP naokfind(SEXP args, int * len, int *naok, int *dup)
 	    /* SETCDR(prev, s = CDR(s)); */
 	    if(dupused++ == 1) warning("DUP used more than once");
 	} else if(TAG(s) == PkgSymbol) {
-	    strcpy(DLLname, CHAR(STRING_ELT(CAR(s), 0)));
+	    p = CHAR(STRING_ELT(CAR(s), 0));
+	    if(strlen(p) > PATH_MAX - 1)
+		error("DLL name is too long");
+	    strcpy(DLLname, p);
 	    if(pkgused++ > 1) warning("PACKAGE used more than once");
 	    /* More generally, this should allow us to process
                any additional arguments and not insist that PACKAGE
                be the last argument.
              */
-             if(s == args) 
-		 args = s = CDR(s);
-             else 
-		 SETCDR(prev, s = CDR(s));
-             continue;
-	} 
-        nargs++;
-	prev = s;
-	s = CDR(s);
+	} else {
+	    nargs++;
+	    prev = s;
+	    s = CDR(s);
+	    continue;
+	}
+	if(s == args) 
+	    args = s = CDR(s);
+	else 
+	    SETCDR(prev, s = CDR(s));
     }
     *len = nargs;
     return args;
 }
 
-static void setDLLname(SEXP s, char *DLLName) {
-  SEXP ss = CAR(s); char *name;
-  if(TYPEOF(ss) != STRSXP || length(ss) != 1)
-    error("PACKAGE argument must be a single character string");
-  name = CHAR(STRING_ELT(ss, 0));
-  /* allow the package: form of the name, as returned by find */
-  if(strncmp(name, "package:", 8) == 0)
-    name += 8;
-  strcpy(DLLname, name);
+static void setDLLname(SEXP s, char *DLLName) 
+{
+    SEXP ss = CAR(s); char *name;
+    if(TYPEOF(ss) != STRSXP || length(ss) != 1)
+	error("PACKAGE argument must be a single character string");
+    name = CHAR(STRING_ELT(ss, 0));
+    /* allow the package: form of the name, as returned by find */
+    if(strncmp(name, "package:", 8) == 0)
+	name += 8;
+    if(strlen(name) > PATH_MAX - 1)
+	error("PACKAGE argument is too long");
+    strcpy(DLLname, name);
 }
 
 static SEXP pkgtrim(SEXP args)
@@ -1173,8 +1205,10 @@ SEXP do_dotCode(SEXP call, SEXP op, SEXP args, SEXP env)
     int dup, havenames, naok, nargs, which;
     DL_FUNC fun;
     SEXP ans, pargs, s;
-    R_toCConverter  *argConverters[65];
+    R_toCConverter  *argConverters[65]; /* the post-call converters back to R objects. */
     R_RegisteredNativeSymbol symbol = {R_C_SYM, {NULL}, NULL};
+    R_NativePrimitiveArgType *checkTypes = NULL;
+    R_NativeArgStyle *argStyles = NULL;
 
     char buf[128], *p, *q, *vmax;
     if (NaokSymbol == NULL || DupSymbol == NULL || PkgSymbol == NULL) {
@@ -1225,6 +1259,9 @@ SEXP do_dotCode(SEXP call, SEXP op, SEXP args, SEXP env)
 	if(symbol.symbol.c->numArgs != nargs)
 	    error("Incorrect number of arguments (%d), expecting %d for %s", 
 		  nargs, symbol.symbol.c->numArgs, buf);
+ 
+        checkTypes = symbol.symbol.c->types;
+        argStyles = symbol.symbol.c->styles;
     }
 
     /* Convert the arguments for use in foreign */
@@ -1234,7 +1271,20 @@ SEXP do_dotCode(SEXP call, SEXP op, SEXP args, SEXP env)
     cargs = (void**)R_alloc(nargs, sizeof(void*));
     nargs = 0;
     for(pargs = args ; pargs != R_NilValue; pargs = CDR(pargs)) {
-	cargs[nargs] = RObjToCPtr(CAR(pargs), naok, dup, nargs + 1, which, buf, argConverters + nargs);
+#ifdef THROW_REGISTRATION_TYPE_ERROR
+        if(checkTypes && !comparePrimitiveTypes(checkTypes[nargs], CAR(pargs), dup)) {
+            /* We can loop over all the arguments and report all the erroneous ones,
+               but then we would also want to avoid the conversions.
+               Also, in the future, we may just attempt to coerce the value
+               to the appropriate type. This is why we pass the checkTypes[nargs]
+               value to RObjToCPtr(). We just have to sort out the ability to 
+               return the correct value which is complicated by dup, etc. */
+  	   error("Wrong type for argument %d in call to %s", nargs+1, buf);
+	}
+#endif
+	cargs[nargs] = RObjToCPtr(CAR(pargs), naok, dup, nargs + 1, 
+				  which, buf, argConverters + nargs, 
+				  checkTypes ? checkTypes[nargs] : 0);
 	nargs++;
     }
 
@@ -1843,7 +1893,9 @@ SEXP do_dotCode(SEXP call, SEXP op, SEXP args, SEXP env)
 	info.functionName = buf;
 	nargs = 0;
 	for (pargs = args ; pargs != R_NilValue ; pargs = CDR(pargs)) {
-            if(argConverters[nargs]) {
+	    if(argStyles && argStyles[nargs] == R_ARG_IN) {
+	        PROTECT(s = R_NilValue);
+	    } else if(argConverters[nargs]) {
                 if(argConverters[nargs]->reverse) {
 		    info.argIndex = nargs;
 		    s = argConverters[nargs]->reverse(cargs[nargs], CAR(pargs), 
@@ -1852,7 +1904,8 @@ SEXP do_dotCode(SEXP call, SEXP op, SEXP args, SEXP env)
 		    s = R_NilValue;
 		PROTECT(s);
 	    } else {
-		PROTECT(s = CPtrToRObj(cargs[nargs], CAR(pargs), which));
+		PROTECT(s = CPtrToRObj(cargs[nargs], CAR(pargs), which, 
+				       checkTypes ? checkTypes[nargs] : TYPEOF(CAR(pargs))));
 		SET_ATTRIB(s, duplicate(ATTRIB(CAR(pargs))));
 		SET_OBJECT(s, OBJECT(CAR(pargs)));
 	    }
@@ -1893,9 +1946,9 @@ SEXP do_dotCode(SEXP call, SEXP op, SEXP args, SEXP env)
 /* FIXME : Must work out what happens here when we replace LISTSXP by
    VECSXP. */
 
-static struct {
-    char *name;
-    SEXPTYPE type;
+static const struct {
+    const char *name;
+    const SEXPTYPE type;
 }
 typeinfo[] = {
     {"logical",	  LGLSXP },
@@ -1990,20 +2043,20 @@ void call_R(char *func, long nargs, void **arguments, char **modes,
     case CPLXSXP:
     case STRSXP:
 	if(nres > 0)
-	    results[0] = RObjToCPtr(s, 1, 1, 0, 0, (const char *)NULL, NULL);
+	    results[0] = RObjToCPtr(s, 1, 1, 0, 0, (const char *)NULL, NULL, 0);
 	break;
     case VECSXP:
 	n = length(s);
 	if (nres < n) n = nres;
 	for (i = 0 ; i < n ; i++) {
-	    results[i] = RObjToCPtr(VECTOR_ELT(s, i), 1, 1, 0, 0, (const char *)NULL, NULL);
+	    results[i] = RObjToCPtr(VECTOR_ELT(s, i), 1, 1, 0, 0, (const char *)NULL, NULL, 0);
 	}
 	break;
     case LISTSXP:
 	n = length(s);
 	if(nres < n) n = nres;
 	for(i=0 ; i<n ; i++) {
-	    results[i] = RObjToCPtr(s, 1, 1, 0, 0, (const char *)NULL, NULL);
+	    results[i] = RObjToCPtr(s, 1, 1, 0, 0, (const char *)NULL, NULL, 0);
 	    s = CDR(s);
 	}
 	break;
