@@ -94,7 +94,7 @@
 #endif
 
 #include "Defn.h"
-#include "R_ext/Callbacks.h"
+#include <R_ext/Callbacks.h>
 
 #define IS_USER_DATABASE(rho)  OBJECT((rho)) && inherits((rho), "UserDefinedDatabase")
 
@@ -616,8 +616,9 @@ void InitGlobalEnv()
     SET_SYMVALUE(install(".BaseNamespaceEnv"), R_BaseNamespace);
     R_BaseNamespaceName = ScalarString(mkChar("base"));
     R_PreserveObject(R_BaseNamespaceName);
-    /**** need to properly initialize the name space */
-    /**** need to create namespace registry */
+    R_NamespaceRegistry = R_NewHashedEnv(R_NilValue);
+    R_PreserveObject(R_NamespaceRegistry);
+    /**** need to properly initialize the base name space */
     /**** need to enter base namespace in registry */
 #endif
 }
@@ -1675,8 +1676,13 @@ static int isMissing(SEXP symbol, SEXP rho)
     else
 	s = symbol;
 
-    if (rho == R_NilValue)  /* is this really the right thing to do? LT */
-	return 0;
+#ifdef EXPERIMENTAL_NAMESPACES
+    if (rho == R_NilValue || rho == R_BaseNamespace)
+	return 0;  /* is this really the right thing to do? LT */
+#else
+    if (rho == R_NilValue)
+	return 0;  /* is this really the right thing to do? LT */
+#endif
 
     vl = findVarLocInFrame(rho, s, NULL);
     if (vl != R_NilValue) {
@@ -2629,7 +2635,7 @@ SEXP R_FindNamespace(SEXP info)
 {
     SEXP fun, expr, val;
     PROTECT(info);
-    fun = install("findNamespaceEnv");
+    fun = install("getNamespace");
     if (findVar(fun, R_GlobalEnv) == R_UnboundValue) { /* not a perfect test */
 	warning("namespaces not abailable; using .GlobalEnv");
 	UNPROTECT(1);
@@ -2643,10 +2649,62 @@ SEXP R_FindNamespace(SEXP info)
     }
 }
 
-SEXP do_useNSDisp(SEXP call, SEXP op, SEXP args, SEXP rho)
+static SEXP checkNSname(SEXP call, SEXP name)
 {
+    switch (TYPEOF(name)) {
+    case STRSXP:
+	if (LENGTH(name) == 1)
+	    name = install(CHAR(STRING_ELT(name, 0)));
+	/* fall through */
+    case SYMSXP: break;
+    default: errorcall(call, "bad name space name");
+    }
+    return name;
+}
+
+SEXP do_regNS(SEXP call, SEXP op, SEXP args, SEXP rho)
+{
+    SEXP name, val;
     checkArity(op, args);
-    R_SetUseNamespaceDispatch(asLogical(CAR(args)));
+    name = checkNSname(call, CAR(args));
+    val = CADR(args);
+    if (findVarInFrame(R_NamespaceRegistry, name) != R_UnboundValue)
+	errorcall(call, "name space already registered");
+    defineVar(name, val, R_NamespaceRegistry);
     return R_NilValue;
 }
+
+SEXP do_unregNS(SEXP call, SEXP op, SEXP args, SEXP rho)
+{
+    SEXP name;
+    int hashcode;
+    checkArity(op, args);
+    name = checkNSname(call, CAR(args));
+    if (findVarInFrame(R_NamespaceRegistry, name) == R_UnboundValue)
+	errorcall(call, "name space not registered");
+    if( !HASHASH(PRINTNAME(name)))
+	hashcode = R_Newhashpjw(CHAR(PRINTNAME(name)));
+    else
+	hashcode = HASHVALUE(PRINTNAME(name));
+    RemoveVariable(name, hashcode, R_NamespaceRegistry);
+    return R_NilValue;
+}
+
+SEXP do_getRegNS(SEXP call, SEXP op, SEXP args, SEXP rho)
+{
+    SEXP name, val;
+    checkArity(op, args);
+    name = checkNSname(call, CAR(args));
+    val = findVarInFrame(R_NamespaceRegistry, name);
+    if (val == R_UnboundValue)
+	return R_NilValue;
+    else
+	return val;
+}
+
+SEXP do_getNSRegistry(SEXP call, SEXP op, SEXP args, SEXP rho)
+{
+    checkArity(op, args);
+    return R_NamespaceRegistry;
+}
 #endif
