@@ -64,11 +64,12 @@ void formatLogical(int *x, int n, int *fieldwidth)
 
     *fieldwidth = 1;
     for(i = 0 ; i < n; i++) {
-	if (x[i] == NA_LOGICAL && *fieldwidth <	 R_print.na_width)
-	    *fieldwidth =  R_print.na_width;
-	else if (x[i] != 0 && *fieldwidth < 4)
+	if (x[i] == NA_LOGICAL) {
+	    if(*fieldwidth < R_print.na_width)
+		*fieldwidth =  R_print.na_width;
+	} else if (x[i] != 0 && *fieldwidth < 4) {
 	    *fieldwidth = 4;
-	else if (x[i] == 0 && *fieldwidth < 5 ) {
+	} else if (x[i] == 0 && *fieldwidth < 5 ) {
 	    *fieldwidth = 5;
 	    break;
 	    /* this is the widest it can be,  so stop */
@@ -145,15 +146,17 @@ void formatInteger(int *x, int n, int *fieldwidth)
  * Using GLOBAL	 R_print.digits	 -- had	 #define MAXDIG R_print.digits
 */
 
-static double tbl[] =
+static const double tbl[] =
 {
     0.e0, 1.e0, 1.e1, 1.e2, 1.e3, 1.e4, 1.e5, 1.e6, 1.e7, 1.e8, 1.e9
 };
 
+#if 0
 static double eps;/* = 10^{- R_print.digits};
 			set in formatReal/Complex,  used in scientific() */
+#endif
 
-static void scientific(double *x, int *sgn, int *kpower, int *nsig)
+static void scientific(double *x, int *sgn, int *kpower, int *nsig, double eps)
 {
     /* for 1 number	 x , return
      *	sgn    = 1_{x < 0}  {0/1}
@@ -215,7 +218,7 @@ void formatReal(double *x, int l, int *m, int *n, int *e, int nsmall)
     int neg, sgn, kpower, nsig;
     int i, naflag, nanflag, posinf, neginf;
 
-    eps = pow(10.0, -(double)R_print.digits);
+    double eps = pow(10.0, -(double)R_print.digits);
 
     nanflag = 0;
     naflag = 0;
@@ -232,7 +235,7 @@ void formatReal(double *x, int l, int *m, int *n, int *e, int nsmall)
 	    else if(x[i] > 0) posinf = 1;
 	    else neginf = 1;
 	} else {
-	    scientific(&x[i], &sgn, &kpower, &nsig);
+	    scientific(&x[i], &sgn, &kpower, &nsig, eps);
 
 	    left = kpower + 1;
 	    sleft = sgn + ((left <= 0) ? 1 : left); /* >= 1 */
@@ -296,7 +299,7 @@ void formatComplex(Rcomplex *x, int l, int *mr, int *nr, int *er,
     int naflag;
     int rnanflag, rposinf, rneginf, inanflag, iposinf;
 
-    eps = pow(10.0, -(double)R_print.digits);
+    double eps = pow(10.0, -(double)R_print.digits);
 
     naflag = 0;
     rnanflag = 0;
@@ -326,7 +329,7 @@ void formatComplex(Rcomplex *x, int l, int *mr, int *nr, int *er,
 	    }
 	    else
 	      {
-		scientific(&(x[i].r), &sgn, &kpower, &nsig);
+		scientific(&(x[i].r), &sgn, &kpower, &nsig, eps);
 
 		left = kpower + 1;
 		sleft = sgn + ((left <= 0) ? 1 : left); /* >= 1 */
@@ -351,7 +354,7 @@ void formatComplex(Rcomplex *x, int l, int *mr, int *nr, int *er,
 	    }
 	    else
 	      {
-		scientific(&(x[i].i), &sgn, &kpower, &nsig);
+		scientific(&(x[i].i), &sgn, &kpower, &nsig, eps);
 
 		left = kpower + 1;
 		sleft = ((left <= 0) ? 1 : left);
