@@ -60,47 +60,42 @@
 #include <Defn.h>
 #include <Rmath.h>
 #include <Print.h>
-#include "R_ext/RS.h"
+#include <R_ext/RS.h>
 #include <Rconnections.h>
+
+#include "RBufferUtils.h"
+
 extern int R_OutputCon; /* from connections.c */
 
+
 #define BUFSIZE 8192  /* used by Rprintf etc */
-static char *Encodebuf=NULL;
+static R_StringBuffer gBuffer = {NULL, 0, BUFSIZE};
+static R_StringBuffer *buffer = &gBuffer; /*XX Add appropriate const here 
+                                            and in the routines that use it. */
 
-static void AllocBuffer(int len)
-{
-    static int bufsize = 0;
-    if(len*sizeof(char) < bufsize) return;
-    len = (len+1)*sizeof(char);
-    if(len < BUFSIZE) len = BUFSIZE;
-    /* Protect against broken realloc */
-    if(Encodebuf) Encodebuf = (char *) realloc(Encodebuf, len);
-    else Encodebuf = (char *) malloc(len);
-    bufsize = len;
-    if(!Encodebuf) {
-	bufsize = 0;
-	error("Could not allocate memory for Encodebuf");
-    }
-}
 
-long Decode2Long(char *p, int *ierr)
+R_size_t R_Decode2Long(char *p, int *ierr)
 {
-    long v = strtol(p, &p, 10);
+    R_size_t v = strtol(p, &p, 10);
     *ierr = 0;
     if(p[0] == '\0') return v;
     /* else look for letter-code ending : */
     if(R_Verbose)
-	REprintf("Decode2Long(): v=%ld\n", v);
-    if(p[0] == 'M') {
-	if((Mega * (double)v) > LONG_MAX) { *ierr = 1; return(v); }
+	REprintf("R_Decode2Long(): v=%ld\n", v);
+    if(p[0] == 'G') {
+	if((Giga * (double)v) > R_SIZE_T_MAX) { *ierr = 1; return(v); }
+	return (Giga*v);
+    }
+    else if(p[0] == 'M') {
+	if((Mega * (double)v) > R_SIZE_T_MAX) { *ierr = 1; return(v); }
 	return (Mega*v);
     }
     else if(p[0] == 'K') {
-	if((1024 * (double)v) > LONG_MAX) { *ierr = 2; return(v); }
+	if((1024 * (double)v) > R_SIZE_T_MAX) { *ierr = 2; return(v); }
 	return (1024*v);
     }
     else if(p[0] == 'k') {
-	if((1000 * (double)v) > LONG_MAX) { *ierr = 3; return(v); }
+	if((1000 * (double)v) > R_SIZE_T_MAX) { *ierr = 3; return(v); }
 	return (1000*v);
     }
     else {
@@ -111,45 +106,45 @@ long Decode2Long(char *p, int *ierr)
 
 char *EncodeLogical(int x, int w)
 {
-    AllocBuffer(0);
-    if(x == NA_LOGICAL) sprintf(Encodebuf, "%*s", w, CHAR(R_print.na_string));
-    else if(x) sprintf(Encodebuf, "%*s", w, "TRUE");
-    else sprintf(Encodebuf, "%*s", w, "FALSE");
-    return Encodebuf;
+    R_AllocStringBuffer(0, buffer);
+    if(x == NA_LOGICAL) sprintf(buffer->data, "%*s", w, CHAR(R_print.na_string));
+    else if(x) sprintf(buffer->data, "%*s", w, "TRUE");
+    else sprintf(buffer->data, "%*s", w, "FALSE");
+    return buffer->data;
 }
 
 char *EncodeInteger(int x, int w)
 {
-    AllocBuffer(0);
-    if(x == NA_INTEGER) sprintf(Encodebuf, "%*s", w, CHAR(R_print.na_string));
-    else sprintf(Encodebuf, "%*d", w, x);
-    return Encodebuf;
+    R_AllocStringBuffer(0, buffer);
+    if(x == NA_INTEGER) sprintf(buffer->data, "%*s", w, CHAR(R_print.na_string));
+    else sprintf(buffer->data, "%*d", w, x);
+    return buffer->data;
 }
 
 char *EncodeReal(double x, int w, int d, int e)
 {
     char fmt[20];
 
-    AllocBuffer(0);
+    R_AllocStringBuffer(0, buffer);
     /* IEEE allows signed zeros (yuck!) */
     if (x == 0.0) x = 0.0;
     if (!R_FINITE(x)) {
-	if(ISNA(x)) sprintf(Encodebuf, "%*s", w, CHAR(R_print.na_string));
+	if(ISNA(x)) sprintf(buffer->data, "%*s", w, CHAR(R_print.na_string));
 #ifdef IEEE_754
-	else if(ISNAN(x)) sprintf(Encodebuf, "%*s", w, "NaN");
+	else if(ISNAN(x)) sprintf(buffer->data, "%*s", w, "NaN");
 #endif
-	else if(x > 0) sprintf(Encodebuf, "%*s", w, "Inf");
-	else sprintf(Encodebuf, "%*s", w, "-Inf");
+	else if(x > 0) sprintf(buffer->data, "%*s", w, "Inf");
+	else sprintf(buffer->data, "%*s", w, "-Inf");
     }
     else if (e) {
 #ifndef Win32
 	if(d) {
 	    sprintf(fmt,"%%#%d.%de", w, d);
-	    sprintf(Encodebuf, fmt, x);
+	    sprintf(buffer->data, fmt, x);
 	}
 	else {
 	    sprintf(fmt,"%%%d.%de", w, d);
-	    sprintf(Encodebuf, fmt, x);
+	    sprintf(buffer->data, fmt, x);
 	}
 #else
 	/* Win32 libraries always use e+xxx format so avoid them */
@@ -163,19 +158,19 @@ char *EncodeReal(double x, int w, int d, int e)
 	if(abs(kp) >= 100) {
 	    if(d) sprintf(fmt,"%%#%d.%de", w, d);
 	    else sprintf(fmt,"%%%d.%de", w, d);
-	    sprintf(Encodebuf, fmt, X);
+	    sprintf(buffer->data, fmt, X);
 	} else {
 	    if(d) sprintf(fmt, "%%#%d.%dfe%%+0%dd", w-ee-3, d, ee+2);
 	    else sprintf(fmt, "%%%d.%dfe%%+0%dd", w-ee-3, d, ee+2);
-	    sprintf(Encodebuf, fmt, x, kp);
+	    sprintf(buffer->data, fmt, x, kp);
 	}
 #endif
     }
     else {
 	sprintf(fmt,"%%%d.%df", w, d);
-	sprintf(Encodebuf, fmt, x);
+	sprintf(buffer->data, fmt, x);
     }
-    return Encodebuf;
+    return buffer->data;
 }
 
 char *EncodeComplex(Rcomplex x, int wr, int dr, int er, int wi, int di, int ei)
@@ -187,13 +182,13 @@ char *EncodeComplex(Rcomplex x, int wr, int dr, int er, int wi, int di, int ei)
     int  flagNegIm = 0;
 #endif
 
-    AllocBuffer(0);
+    R_AllocStringBuffer(0, buffer);
     /* IEEE allows signed zeros; strip these here */
     if (x.r == 0.0) x.r = 0.0;
     if (x.i == 0.0) x.i = 0.0;
 
     if (ISNA(x.r) || ISNA(x.i)) {
-	sprintf(Encodebuf, "%*s%*s", R_print.gap, "", wr+wi+2,
+	sprintf(buffer->data, "%*s%*s", R_print.gap, "", wr+wi+2,
 		CHAR(R_print.na_string));
     }
     else {
@@ -201,7 +196,7 @@ char *EncodeComplex(Rcomplex x, int wr, int dr, int er, int wi, int di, int ei)
 	if(er) efr = "e"; else efr = "f";
 	if(ei) efi = "e"; else efi = "f";
 	sprintf(fmt,"%%%d.%d%s%%+%d.%d%si", wr, dr, efr, wi, di, efi);
-	sprintf(Encodebuf, fmt, x.r, x.i);
+	sprintf(buffer->data, fmt, x.r, x.i);
 #else
 
 	/* EncodeReal returns pointer to static storage so copy */
@@ -216,12 +211,12 @@ char *EncodeComplex(Rcomplex x, int wr, int dr, int er, int wi, int di, int ei)
 	Im = Calloc(strlen(tmp)+1, char);
 	strcpy(Im, tmp);
 
-	sprintf(Encodebuf, "%s%s%si", Re, flagNegIm ? "-" : "+", Im);
+	sprintf(buffer->data, "%s%s%si", Re, flagNegIm ? "-" : "+", Im);
 
 	Free(Re); Free(Im);
     }
 #endif
-    return Encodebuf;
+    return buffer->data;
 }
 
 	/* There is a heavy ASCII emphasis here */
@@ -293,8 +288,8 @@ char *EncodeString(char *s, int w, int quote, int right)
 	i = Rstrlen(s, quote);
     }
     
-    AllocBuffer((i+2 >= w)?(i+2):w); /* +2 allows for quotes */
-    q = Encodebuf;
+    R_AllocStringBuffer((i+2 >= w)?(i+2):w, buffer); /* +2 allows for quotes */
+    q = buffer->data;
     if(right) { /* Right justifying */
 	b = w - i - (quote ? 2 : 0);
 	for(i=0 ; i<b ; i++) *q++ = ' ';
@@ -346,11 +341,11 @@ char *EncodeString(char *s, int w, int quote, int right)
     if(quote) *q++ = quote;
     if(!right) { /* Left justifying */
 	*q = '\0';
-	b = w - strlen(Encodebuf);
+	b = w - strlen(buffer->data);
 	for(i=0 ; i<b ; i++) *q++ = ' ';
     }
     *q = '\0';
-    return Encodebuf;
+    return buffer->data;
 }
 
 char *EncodeElement(SEXP x, int indx, int quote)
@@ -381,19 +376,19 @@ char *EncodeElement(SEXP x, int indx, int quote)
 		      w, d, e, wi, di, ei);
 	break;
     }
-    return Encodebuf;
+    return buffer->data;
 }
 
 char *Rsprintf(char *format, ...)
 {
     va_list(ap);
 
-    AllocBuffer(0); /* unsafe, as assuming length, but all internal
-		       uses are for a few characters */
+    R_AllocStringBuffer(0, buffer); /* unsafe, as assuming length, but all internal
+ 		                       uses are for a few characters */
     va_start(ap, format);
-    vsprintf(Encodebuf, format, ap);
+    vsprintf(buffer->data, format, ap);
     va_end(ap);
-    return Encodebuf;
+    return buffer->data;
 }
 
 void Rprintf(char *format, ...)
@@ -496,11 +491,14 @@ void VectorIndex(int i, int w)
 void MatrixColumnLabel(SEXP cl, int j, int w)
 {
     int l;
+    SEXP tmp;
 
-    if (!isNull(cl)) {
-	l = Rstrlen(CHAR(STRING_ELT(cl, j)), 0);
+    if (!isNull(cl)) { 
+        tmp = STRING_ELT(cl, j);
+	if(tmp == NA_STRING) l = R_print.na_width_noquote;
+	else l = Rstrlen(CHAR(tmp), 0);
 	Rprintf("%*s%s", w-l, "",
-		EncodeString(CHAR(STRING_ELT(cl, j)), l, 0, Rprt_adj_left));
+		EncodeString(CHAR(tmp), l, 0, Rprt_adj_left));
     }
     else {
 	Rprintf("%*s[,%ld]", w-IndexWidth(j+1)-3, "", j+1);
@@ -510,11 +508,14 @@ void MatrixColumnLabel(SEXP cl, int j, int w)
 void RightMatrixColumnLabel(SEXP cl, int j, int w)
 {
     int l;
+    SEXP tmp;
 
-    if (!isNull(cl)) {
-	l = Rstrlen(CHAR(STRING_ELT(cl, j)), 0);
+    if (!isNull(cl)) { 
+        tmp = STRING_ELT(cl, j);
+	if(tmp == NA_STRING) l = R_print.na_width_noquote;
+	else l = Rstrlen(CHAR(tmp), 0);
 	Rprintf("%*s", R_print.gap+w,
-		EncodeString(CHAR(STRING_ELT(cl, j)), l, 0, Rprt_adj_right));
+		EncodeString(CHAR(tmp), l, 0, Rprt_adj_right));
     }
     else {
 	Rprintf("%*s[,%ld]%*s", R_print.gap, "", j+1, w-IndexWidth(j+1)-3, "");
@@ -524,11 +525,14 @@ void RightMatrixColumnLabel(SEXP cl, int j, int w)
 void LeftMatrixColumnLabel(SEXP cl, int j, int w)
 {
     int l;
+    SEXP tmp; 
 
-    if (!isNull(cl)) {
-	l = Rstrlen(CHAR(STRING_ELT(cl, j)), 0);
+    if (!isNull(cl)) { 
+        tmp= STRING_ELT(cl, j);
+	if(tmp == NA_STRING) l = R_print.na_width_noquote;
+	else l = Rstrlen(CHAR(tmp), 0);
 	Rprintf("%*s%s%*s", R_print.gap, "",
-		EncodeString(CHAR(STRING_ELT(cl, j)), l, 0, Rprt_adj_left), w-l, "");
+		EncodeString(CHAR(tmp), l, 0, Rprt_adj_left), w-l, "");
     }
     else {
 	Rprintf("%*s[,%ld]%*s", R_print.gap, "", j+1, w-IndexWidth(j+1)-3, "");
@@ -538,11 +542,14 @@ void LeftMatrixColumnLabel(SEXP cl, int j, int w)
 void MatrixRowLabel(SEXP rl, int i, int rlabw, int lbloff)
 {
     int l;
+    SEXP tmp;
 
-    if (!isNull(rl)) {
-	l = Rstrlen(CHAR(STRING_ELT(rl, i)), 0);
+    if (!isNull(rl)) { 
+        tmp= STRING_ELT(rl, i);
+	if(tmp == NA_STRING) l = R_print.na_width_noquote;
+	else l = Rstrlen(CHAR(tmp), 0);
 	Rprintf("\n%*s%s%*s", lbloff, "",
-		EncodeString(CHAR(STRING_ELT(rl, i)), l, 0, Rprt_adj_left),
+		EncodeString(CHAR(tmp), l, 0, Rprt_adj_left), 
 		rlabw-l-lbloff, "");
     }
     else {
