@@ -511,7 +511,7 @@ static SEXP scanVector(SEXPTYPE type, int maxitems, int maxlines,
 	break;
     }
     UNPROTECT(1);
-    if(strBuf.data) free(strBuf.data);
+    R_FreeStringBuffer(&strBuf);
     return bns;
 }
 
@@ -666,7 +666,7 @@ static SEXP scanFrame(SEXP what, int maxitems, int maxlines, int flush,
 	SET_VECTOR_ELT(ans, i, new);
     }
     UNPROTECT(1);
-    if(buf.data) free(buf.data);
+    R_FreeStringBuffer(&buf);
     return ans;
 }
 
