@@ -30,3 +30,14 @@ C These now all call C functions via F77_NAME(.) in ./print.c :
       if(nc .lt. 0) nc = len(label)
       call dblep0(label, nc, data, ndata)
       end
+
+C R-only Fortran versions of error and warning
+      subroutine rexit(msg)
+      character*(*) msg
+      call rexitc(msg, len(msg))
+      end
+
+      subroutine rwarn(msg)
+      character*(*) msg
+      call rwarnc(msg, len(msg))
+      end
