@@ -366,9 +366,9 @@ RxmlNanoHTTPScanProxy(const char *URL)
         xmlFree(proxy);
 	proxy = NULL;
     }
-    if (proxyPort != 0) {
+    /*if (proxyPort != 0) {
 	proxyPort = 0;
-    }
+	}*/
     if (URL == NULL)
 	RxmlMessage(0, "Removing HTTP proxy info");
     else
