@@ -81,3 +81,33 @@ t1 <- ts(0:100)
 plot(t1, log = "y")
 plot(cbind(t1, 10*t1, t1 - 4), log="y", plot.type = "single")
 stopifnot(par("usr")[4] > 3) # log10: ylim[2] = 1000
+
+
+## This one needs to be looked at.
+## lty = "blank" killed the fill colour too.
+plot(1:10, type="n")
+polygon(c(1, 3, 3, 1), c(1, 1, 3, 3), col="yellow", border="red", lty="blank")
+rect(6, 6, 10, 10,  col="blue", border="red", lty="blank")
+## in 1.5.0 all omit the fill colours.
+data(trees)
+with(trees, symbols(Height, Volume, circles=Girth/24, inches=FALSE,
+                    lty="blank", bg="blue"))
+## in 1.5.0 ignored the lty.
+
+## axis() and par(mgp < 0) {keep this example S+ compatible!}:
+lt <- if(is.R()) "31" else 2
+x <- seq(-2,3, len=1001)
+op <- par(tck= +0.02, mgp = -c(3,2,0))
+plot(x, x^2 - 1.2, xaxt = "n", xlab="", type ='l', col = 2,
+     main = "mgp < 0: all ticks and labels inside `frame'")
+x <- -2:3
+lines(x, x^2 - 1.2, type ="h", col = 3, lwd=3)
+axis(1, pos = 0, at=-1:1, lty = lt, col=4)## col & lty work only from R 1.6
+par(op)
+axis(1, pos = 0, at=c(-2,2,3), lty = lt, col=4)
+mtext(side=1,"note the x-ticks on the other side of the bars")
+
+## explicit xlab and ylab for non-1D  plot.table():
+data(UCBAdmissions)
+plot(UCBAdmissions)# default x- and y-lab
+plot(UCBAdmissions, xlab = "x label", ylab = "YY")# wrong in 1.5.1
